//*****************************************************************************
//*
//*
//*		LabelCtrl.cpp
//*
//*
//*****************************************************************************
//
//	Copyright (C) Anton Zechner 2008
//
//	LabelCtrl is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
//	Sourcecode which use LabelCtrl must be published. Commercial users
//	must published their code too, or make an licence agreement with me.
//
//
//	LabelCtrl wird unter GNU GENERAL PUBLIC LICENSE (GPL) vertreiben.
//	Sourcecode welcher LabelCtrl verwendet muss verffentlicht werden.
//	Komerzielle Nutzer mssen ihren Code ebenfalls verffentlichen, oder
//	eine Nutzungsvereinbarung mit mir treffen.
//
//	az_software@inode.at
//
//
//	Funktion:
//
//		Ein Label-Fenster schickt an sein Elternfenster eine WM_NOTIFY
//		Nachricht diese kann ber ON_NOTIFY(CLW_SELCHANDED,uId,MemberFunktion)
//		abgefangen werden.
//
//		MemberFunktion ist vom Typ
//
//			OnSelChanged(NMHDR *pNmHdr, LRESULT *pResult)
//				{
//				LabelNotify	*pNotify=(LabelNotify*)pNmHdr;
//				...
//				}
//
//			pNotify->hdr.idFrom		 : Ist die ID des LabelCtrl
//			pNotify->selct			 : Ist die neu gewhlte Lasche
//			pNotify->param			 : Ist der Wert von Userdata in LabelCtrl
//			pNotify->selid			 : Ist die neu gewhlte Lasche
//			pNotify->select			 : Ist die Nummer des gerade gewhlte Eintrags
//			pNotify->selwnd			 : Ist das Fenster-Handle des gewhlten Eintrags
//
//		Ist einer Lasche ein Fenster zu geordnet so wird das Fenster
//		sichtbar bzw. unsichtbar geschalten.
//
//
//	Fenster erzeugen:
//
//		m_cLabelCtrl.Create(iScanner,0,"Daten",sRect,this,CLW_FRAMETOP|CLW_CHILDSCROLL|CLW_CHILDSCROLLOFF|CLW_WHITELINE,12345);
//		m_cLabelCtrl.SetChildRect(sRect.left,sRect.top-200,sRect.right,sRect.top);
//		m_cLabelCtrl.SetWindow(iScanner,m_aListWnd);
//		m_cLabelCtrl.ShowWindow(SW_SHOW);
//		Achtung: GWL_USERDATA darf bei den einzelnen Fenstern nicht verndert werden.
//
//
//	Grennderung des Elternfensters
//
//		m_cLabelCtrl.SetChildRect(0,0     ,iSizeX,       iWidth);
//		m_cLabelCtrl.MoveWindow  (0,iWidth,iSizeX,iSizeY-iWidth);
//		m_cLabelCtrl.UpdateChildPos();
//
//
//	Funktionen:
//
// 		int   GetSel      ()						: Liest aktuelle Auswahl
// 		int	  SetSel      (UINT     uLabel)			: Setzt aktuelle Auswahl
// 		int	  SetSel      (CWnd    *pWnd  )			: Setzt aktuelle Auswahl mit CWnd Zeiger
// 		int	  GetSelId    (UINT     uId   )			: Liest ID des aktuellen Fensters
// 		int	  SetSelId    (UINT 	uId   )			: Setzt aktuelle Auswahl mit ID
// 		int	  SetWidth    (int      iWidth)			: Laschenbreiten auf einmal setzen
// 		int	  SetWidth    (int,int  iWidth,BOOL)	: Einzelne Laschenbreite setzen
// 		int	  SetWidth    (int,int *iWidth,BOOL)	: Laschenbreiten ber Feld setzen
// 		int	  GetWidth    (UINT     uLabel  )		: Laschenbreite lesen
// 		int	  GetLabel    (UINT     uId     )		: Lasche mit uId finden
// 		int	  GetLabel    (CWnd    *pWnd    )		: Lasche mit Fenster finden
// 		int	  GetCount    (                 )		: Anzahl der Laschen lesen
// 		int	  SetCount    (int         ,BOOL)		: Anzahl der Laschen setzen
// 		char *GetText     (UINT       uLabel)		: Alle Laschennamen lesen
// 		int	  SetText     (char      **,BOOL)		: Alle Laschennamen setzen mit Kette
// 		int	  SetText     (char       *,BOOL)		: Alle Laschennamen setzen
// 		int	  SetText     (int,char  **,BOOL)		: Alle Laschennamen setzen mit Feld
// 		int	  SetText     (int,char   *,BOOL)  		: Einen Laschennamen setzen
// 		CWnd *GetWnd      (UINT  uLabel     )		: Fenster mit Nummer finden
// 		CWnd *GetWndId    (UINT	 uId        )		: Fenster mit uId finden
// 		int	  SetWindow   (int,CWnd    &pWnd)		: Einer Lasche ein Fenster zuordnen
// 		int	  SetWindow   (int,CWnd    *pWnd)		: Allen Laschen Fenster zuordnen
// 		int	  SetWindow   (int,CWnd   **pWnd)		: Allen Laschen Fenster zuordnen
// 		int	  SetFlags    (int iFlags       )		: Zeichenmodus fr Lasche
// 		void  SetChildRect(RECT &sRect      )		: Child-Bereich einstellen
// 		void  SetChildRect(int,int,int,int  ) 		: Child-Bereich einstellen
// 		void  Update      (                 )		: Scrollzustand Updaten
//
#include	"LabelCtrl.h"


#ifdef     _DEBUG
#undef      THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

#ifdef	   _MSC_VER
#pragma		warning( disable :4996 )
#endif

#define 	ComineRgb1(uColor1,uColor2)	(((uColor1>>2)&0x003F3F3F)+((uColor2>>2)&0x003F3F3F)*3)
#define 	ComineRgb2(uColor1,uColor2)	(((uColor1>>3)&0x001F1F1F)+((uColor2>>3)&0x001F1F1F)*7)

#define 	F10		0
#define 	F11		1
#define 	F12		2
#define 	F13		3
#define 	F14		4
#define 	F15		5
#define 	F16		6
#define 	F17		7
#define 	F18		8

#define 	F20		9
#define 	F21		10
#define 	F22		11
#define 	F23		12
#define 	F24		13
#define 	F25		14
#define 	F26		15
#define 	F27		16
#define 	F28		17

#define 	F30		18
#define 	F31		19
#define 	F32		20
#define 	F33		21
#define 	F34		22
#define 	F35		23

#define 	F40		24
#define 	F41		25
#define 	F42		26
#define 	F43		27
#define 	F44		28
#define 	F45		29

const BYTE aImgData[20*34] =
		{
		F31,F11,F17,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F18,F12,F41, 0,
		F34,F11,F14,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F15,F12,F44, 0,
		F30,F31,F11,F17,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F18,F12,F41,F40, 0,
		F30,F34,F11,F14,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F10,F15,F12,F44,F40, 0,
		F30,F30,F31,F11,F17,F10,F10,F10,F10,F10,F10,F10,F10,F10,F18,F12,F41,F40,F40, 0,
		F30,F30,F34,F11,F14,F10,F10,F10,F10,F10,F10,F10,F10,F10,F15,F12,F44,F40,F40, 0,
		F30,F30,F30,F31,F11,F17,F10,F10,F10,F10,F10,F10,F10,F18,F12,F41,F40,F40,F40, 0,
		F30,F30,F30,F34,F11,F14,F10,F10,F10,F10,F10,F10,F10,F15,F12,F44,F40,F40,F40, 0,
		F30,F30,F30,F30,F31,F11,F17,F10,F10,F10,F10,F10,F18,F12,F41,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F34,F11,F14,F10,F10,F10,F10,F10,F15,F12,F44,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F31,F11,F17,F10,F10,F10,F18,F12,F41,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F34,F11,F14,F10,F10,F10,F15,F12,F44,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F31,F11,F17,F10,F18,F12,F41,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F34,F11,F14,F10,F15,F12,F44,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F30,F31,F11,F16,F12,F41,F40,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F30,F34,F11,F16,F12,F44,F40,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F30,F30,F33,F13,F43,F40,F40,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F30,F30,F33,F13,F43,F40,F40,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F30,F35,F21,F26,F22,F45,F40,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F30,F32,F21,F26,F22,F42,F40,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F35,F21,F24,F20,F25,F22,F45,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F30,F32,F21,F27,F20,F28,F22,F42,F40,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F35,F21,F24,F20,F20,F20,F25,F22,F45,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F30,F32,F21,F27,F20,F20,F20,F28,F22,F42,F40,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F35,F21,F24,F20,F20,F20,F20,F20,F25,F22,F45,F40,F40,F40,F40, 0,
		F30,F30,F30,F30,F32,F21,F27,F20,F20,F20,F20,F20,F28,F22,F42,F40,F40,F40,F40, 0,
		F30,F30,F30,F35,F21,F24,F20,F20,F20,F20,F20,F20,F20,F25,F22,F45,F40,F40,F40, 0,
		F30,F30,F30,F32,F21,F27,F20,F20,F20,F20,F20,F20,F20,F28,F22,F42,F40,F40,F40, 0,
		F30,F30,F35,F21,F24,F20,F20,F20,F20,F20,F20,F20,F20,F20,F25,F22,F45,F40,F40, 0,
		F30,F30,F32,F21,F27,F20,F20,F20,F20,F20,F20,F20,F20,F20,F28,F22,F42,F40,F40, 0,
		F30,F35,F21,F24,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F25,F22,F45,F40, 0,
		F30,F32,F21,F27,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F28,F22,F42,F40, 0,
		F35,F21,F24,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F25,F22,F45, 0,
		F32,F21,F27,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F20,F28,F22,F42, 0,
		};


typedef struct
		{
		BITMAPINFOHEADER	sHeader;
		DWORD				aColors[30];
		}SplitterImage;

const	BYTE	aSectorColors[4*8] =
		{
		0,0,0,0,							// l:nichts		r=nichts
		1,0,0,1,							// l:nichts		r=grau
		2,0,0,2,							// l:nichts		r=weis

		1,0,1,0,							// l:grau		r=nichts
		1,0,1,1,							// l:grau		r=grau
		2,0,1,2,							// l:grau		r=weis

		2,0,2,0,							// l:weis		r=nichts
		2,0,2,1,							// l:weis		r=grau
		};

const	BYTE	aLineColors[6*8] =
		{
		0,0,0,0,0,0,						// l:nichts		r=nichts
		0,3,3,3,1,3,						// l:nichts		r=grau
		0,3,3,3,2,3,						// l:nichts		r=weis

		3,0,3,1,3,3,						// l:grau		r=nichts
		3,3,3,3,1,3,						// l:grau		r=grau
		3,3,3,3,2,3,						// l:grau		r=weis

		3,0,3,2,3,3,						// l:weis		r=nichts
		3,3,3,2,3,3,						// l:weis		r=grau
		};

inline	LONG    GetId     (HWND  hWnd               ){return          ::GetWindowLong   (         hWnd,GWL_ID      );}
inline	LONG    GetId     (CWnd *pWnd               ){return          ::GetWindowLong   ( pWnd->m_hWnd,GWL_ID      );}
inline	LPARAM  GetData   (CWnd *pWnd               ){return          ::GetWindowLongPtr( pWnd->m_hWnd,GWL_USERDATA);}
inline	LONG    SetStyle  (CWnd *pWnd,LONG    lStyle){return          ::SetWindowLong   ( pWnd->m_hWnd,GWL_STYLE  ,lStyle);}
inline	WNDPROC GetWndProc(CWnd *pWnd               ){return (WNDPROC)::GetWindowLongPtr( pWnd->m_hWnd,GWL_WNDPROC );}
inline	WNDPROC SetWndProc(HWND  hWnd,WNDPROC pProc ){return (WNDPROC)::SetWindowLongPtr(         hWnd,GWL_WNDPROC,(LPARAM)pProc );}
inline	WNDPROC SetWndProc(CWnd *pWnd,WNDPROC pProc ){return (WNDPROC)::SetWindowLongPtr( pWnd->m_hWnd,GWL_WNDPROC,(LPARAM)pProc );}


//*****************************************************************************
//*
//*		SwapRGB
//*
//*****************************************************************************
//  Tauscht eine RGB-Farbe in eine GBR-Farbe

#ifdef	   _MSC_VER
#pragma		warning( push )
#pragma		warning( disable :4035 )
#else
#ifndef		NO_ASM
#define		NO_ASM
#endif
#endif

static inline unsigned SwapRGB(unsigned uColor)
{
#ifdef		NO_ASM

	return ((uColor&0x00FF0000)>>16)|(uColor&0xFF00FF00)|((uColor&0x000000FF)<<16);

#elif defined(_WIN64) || !defined(_M_IX86)

	return _rotr(_byteswap_ulong(uColor),8);

#else

_asm	{
		mov		eax,uColor
		bswap	eax;				;// Tausche Bytes in EAX
		ror		eax,8
		}

#endif
}

#ifdef	   _MSC_VER
#pragma		warning( pop )
#endif

//*****************************************************************************
//*
//*		Konstanten
//*
//*****************************************************************************

#define 	TEXT_ADD		4									// Verbreiterung bei Textbreite
#define 	WIDTH_SMOVE		4									// Breite Schroll-Move Feld
#define 	WIDTH_MIN		10									// Minimale Breite
#define 	WIDTH_MAX		256									// Maximale Breite
#define 	TIMER_LEN		256									// Timerzeit in ms
#define 	TIMER_BT		0xABAA								// Timer fr Buttons
#define 	TIMER_ID		0xABAB								// Timer uId
#define 	SCROLL_SEL		0xABAC								// Scrollleisten ID (Select)
#define 	SCROLL_SCR		0xABAD								// Scrollleisten ID (Scroll)
#define 	WM_LUPDATE		0x2D91								// Fenster erneuern

HCURSOR 	CLabelCtrl::m_hCursorSize	= 0;					// Cursor zum Breiten ndern
HCURSOR 	CLabelCtrl::m_hCursorArrow	= 0;					// Normaler Cursor
HFONT	 	CLabelCtrl::m_hTextFont		= 0;					// Schrift fr Anzeige
int			CLabelCtrl::m_iInitDone		= 0;					// Anzahl der Fenster
int			CLabelCtrl::m_iScrollHeight = 0;					// Hhe   einer Scroll Taste
int			CLabelCtrl::m_iScrollWidth  = 0;					// Breite einer Scroll Taste


//*****************************************************************************
//*
//*		Nachrichten
//*
//*****************************************************************************

BEGIN_MESSAGE_MAP(CLabelCtrl, CWnd)
	//{{AFX_MSG_MAP(CLabelCtrl)
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	ON_WM_DESTROY()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_SYSCOLORCHANGE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
CLabelCtrl::CLabelCtrl()
{
OSVERSIONINFO	sInfo;
WNDCLASS		sClass;
LOGFONT			sLog;



	if(!m_iInitDone)											// Initialiesierung der Fensterklasse
		{
		m_iInitDone		= 1;
		m_hCursorSize	= LoadCursor(0,IDC_SIZEWE);
		m_hCursorArrow	= LoadCursor(0,IDC_ARROW );
		m_iScrollWidth	= GetSystemMetrics(SM_CXHSCROLL)*2;
		m_iScrollHeight	= GetSystemMetrics(SM_CYHSCROLL);

		if(m_iScrollHeight<16)m_iScrollHeight=16;

		sClass.style			= CS_HREDRAW|CS_VREDRAW;		// Eine neue Klasse registreiern
		sClass.lpfnWndProc		= ::DefWindowProc;
		sClass.cbClsExtra		= 0;
		sClass.cbWndExtra		= 0;
		sClass.hInstance		= AfxGetInstanceHandle();;
		sClass.hIcon			= NULL;
		sClass.hCursor			= NULL;
		sClass.hbrBackground	= NULL;
		sClass.lpszMenuName		= NULL;
		sClass.lpszClassName	= _T("LabelCtrl");

		if(!AfxRegisterClass(&sClass))
			{
			AfxThrowResourceException();
			}

		sInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx(&sInfo);

		if(sInfo.dwMajorVersion>=6)								// Ist es kein XP
			{													
			SystemParametersInfo(SPI_GETICONTITLELOGFONT,sizeof(sLog),&sLog,0);

			if(sLog.lfHeight<0)
				{
				sLog.lfHeight+=2;
				if(7-sLog.lfHeight>m_iScrollHeight)sLog.lfHeight=7-m_iScrollHeight;
				if(  sLog.lfHeight>-13            )sLog.lfHeight=-13;
				}
			else{
				sLog.lfHeight-=2;
				if(7+sLog.lfHeight>m_iScrollHeight)sLog.lfHeight=m_iScrollHeight-7;
				if(  sLog.lfHeight< 13            )sLog.lfHeight=13;
				}

			m_hTextFont	= CreateFontIndirect(&sLog);
			}

		if(!m_hTextFont)
			{
			m_hTextFont = (HFONT)GetStockObject(ANSI_VAR_FONT);
			}
		}


	m_sRectChild.SetRectEmpty();								// Child Fenster

	m_iSizeX			=  0;									// Aktuelle Fenstergre
	m_iSizeY			=  0;									//			*
	m_iLabelWidth		=  0;									//			*
	m_iLabelsPosX		=  0;									// X-Position der Laschen
	m_iLabelsPosY		=  0;									// Y-Position der Laschen
	m_pWidths	 		=  0;									// Feld mit Breiten
	m_pCalHeight		=  0;									// Korrigierte Hhe fr Fenster
	m_pTextSizes 		=  0;									// Feld mit Text-Breiten
	m_pTextList	 		=  0;									// Feld mit Namen
	m_pSubProcs			=  0;									// Callbacks fr Fenster
	m_iCount	 		=  0;									// Anzahl der Labels
	m_hBitmap			=  0;									// Bitmaps fr Splitter
	m_pWndList			=  0;
	m_iInit	 			=  0;
	m_iFlags    		=  0;

	m_iScrollOff		=  0;									// Ist Scrollbar abgeschalten
	m_iScrollOld		=  0;									// Alte Scrollbar Breite
	m_iScrollMove		=  0;									// Wird Scroll Breite eingesttellt
	m_iScrollHalf		=  1;									// Scrollbar auf halbe Breite
	m_iStartOffset		=  0;									// Position des erstes Lasche
	m_iStartLabel		=  0;									// Erstes gezeichnetes Lasche
	m_iSelect			=  0;									// Aktuelle Lasche
	m_pChildWnd			=  0;									// Zeiger auf Child Fenster
	m_pChildParent		=  0;									// Zeiger auf Child Elternfenster
	m_iScrollMode		=  0;									// Ist der modus fr's Fenster
	m_iCalibrate		=  0;									// Aktuelle Korrigierte Hhe
	m_iLastEnable		= -1;									// Letzte Einstellungen fr Ebable-Step-Bar
	m_iLastButtons		= -1;									// Letzte Einstellungen fr Step-Bar
	m_iLastButtonsPos	= -1;									// Letzte Position fr Step-Bar
	m_uColorLabels		=  0xFFFFFFFF;							// Farben fr die Laschen (0xFFFFFFFF=default)
	m_uColorLabUse		=  0x00C0C0C0;							// Farben fr die Laschen (zum Zeichen)
	m_uColorLabelW		=  0xFFFFFFFF;							// Farben fr die weise Lasche (0xFFFFFFFF=default)
	m_uColorLabWin		=  0x00C0C0C0;							// Farben fr die weise Lasche (zum Zeichen)
	m_uColorText		=  0xFFFFFFFF;							// Farben fr den Text (0xFFFFFFFF=default)
	m_uColorTextUse		=  0x00000000;							// Farben fr den Text (zum Zeichen)
	m_uColorBg			=  0xFFFFFFFF;							// Farben fr den Hintergrund (0xFFFFFFFF=default)
	m_uColorBgUse		=  0x00C0C0C0;							// Farben fr den Hintergrund (zum Zeichen)
	m_uColorFrame		=  0xFFFFFFFF;							// Farben fr die Linien (0xFFFFFFFF=default)
	m_uColorFrUse		=  0x00000000;							// Farben fr die Linien (zum Zeichen)

	m_iSplitterHeight	=  m_iScrollHeight;
	m_iSplitterWidth	= (m_iScrollHeight/4)*2+3;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CLabelCtrl::~CLabelCtrl()
{

	Destroy();

}

//*****************************************************************************
//*
//*		Create
//*
//*****************************************************************************
//	Erzeugen eines Label-Fenster
//	uLabels		: Anzahl der Laschen
//	pWidths		: Breiten der einzelnen Laschen (bei 0 die Textlnge verwenden)
//	pTextList   : Namen der Laschen
//	pWnd		: Angedocke Fenster
//	sRect		: Fenstergre und Position
//	pParent		: Elternfenster
//	dwStyle		: Fensterstile
//	uId			: Fenster-ID-Nummer
//	iMode		: Bit 0==1 --> pWidths   ist Zeigerfeld
//				: Bit 1==1 --> pTextList ist Zeigerfeld
BOOL CLabelCtrl::Create(UINT uLabels,int *pWidths,LPCTSTR *pTextList,CWnd **pWnd,const RECT &sRect,CWnd *pParent,DWORD dwStyle,UINT uId,int iMode)
{
CRect	sTemp;
int		iSel;


	ASSERT_VALID(pParent);

	if(!pWidths  )iMode&=~1;
	if(!pTextList)iMode&=~2;

				  SetCount(uLabels);
	if(iMode&0x01)SetWidth(uLabels ,pWidths  ,0);
	else	      SetWidth((int)    pWidths    );
	if(iMode&0x02)SetText (uLabels ,pTextList,0);
	else	      SetText ((LPCTSTR)pTextList,0);
	if(iMode&0x10)dwStyle|=WS_CHILD|WS_CLIPSIBLINGS|WS_CLIPCHILDREN;

	m_iFlags = dwStyle&0xFFFF;

	sTemp.SetRect(sRect.left,sRect.top,sRect.left+m_iScrollWidth,sRect.top+m_iScrollHeight);

	if(m_iFlags&CLW_FRAMETOP  ){sTemp.top ++;sTemp.bottom++;}
	if(m_iFlags&CLW_FRAMERIGHT){sTemp.left++;sTemp.right ++;}

	if(!CWnd::Create(_T("LabelCtrl"),NULL,dwStyle,sRect,pParent,uId))return FALSE;

	m_sWndData.hWnd			= 0;								// Fenster Handle
	m_sWndData.iStop		= 0;								// Ist die Fensterfunktion bereits aufgerufen
	m_sWndData.pWndProc		= NULL;								// Alte Fensterprozedur
	m_sWndData.pLabelCtrl	= this;								// This-Zeiger


	m_sButtons.Create(SBS_HORZ|WS_VISIBLE|WS_CLIPCHILDREN|WS_CLIPSIBLINGS,sTemp,this,SCROLL_SEL);
	m_sScroll .Create(SBS_HORZ|WS_VISIBLE|WS_CLIPCHILDREN|WS_CLIPSIBLINGS,sTemp,this,SCROLL_SCR);
	sTemp.left=0;
	sTemp.right=0;
	m_sMoveWnd.Create(_T(""),SS_GRAYRECT,sTemp,this);

	CreateSplitter();



	m_iInit				=  1;
	m_iScrollOff		=  0;
	m_iLastEnable		= -1;
	m_iLastButtons		= -1;
	m_iLastButtonsPos	= -1;

	if(m_iSizeX && m_iSizeY)
		{
		OnSize(0,m_iSizeX,m_iSizeY);
		}


	SetButtons();

	if(m_iFlags&CLW_CHILD)
		{
		SetTimer(TIMER_ID,TIMER_LEN,0);
		}

	if(pWnd)													// Die Fenster aus der Liste eintragen
		{
		SetWindow(uLabels,pWnd);
		}


	iSel = m_iSelect;
	       m_iSelect = -1;

	SetSel(iSel);


	if(sRect.right >sRect.left)
	if(sRect.bottom>sRect.top )
		{
		OnSize(0,sRect.right-sRect.left,sRect.bottom-sRect.top);
		}


return TRUE;
}


//*****************************************************************************
//*
//*		Destroy
//*
//*****************************************************************************
void CLabelCtrl::Destroy()
{
int		i;


	if(m_pTextList)												// Texte lschen
		{
		for(i=0;i<m_iCount;i++)
			{
			if(   !m_pTextList[i])continue;
			delete m_pTextList[i];
			}

		delete m_pTextList;

		m_pTextList = 0;
		}

	if(m_pWidths   ){delete m_pWidths;		 m_pWidths   =0;}	// Feld mit Breiten
	if(m_pCalHeight){delete m_pCalHeight;	 m_pCalHeight=0;}	// Korrigierte Hhe fr Fenster
	if(m_pTextSizes){delete m_pTextSizes;	 m_pTextSizes=0;}	// Feld mit Text-Breiten
	if(m_pSubProcs ){delete m_pSubProcs;	 m_pSubProcs =0;}	// Callbacks fr Fenster
	if(m_pWndList  ){delete m_pWndList;		 m_pWndList  =0;}	// Feld mit Fenstern
	if(m_hBitmap   ){DeleteObject(m_hBitmap);m_hBitmap   =0;}	// Bitmaps fr Splitter

	m_iCount  = 0;
	m_iSelect = 0;




}

//*****************************************************************************
//*
//*		SetButtons
//*
//*****************************************************************************
void CLabelCtrl::SetButtons()
{
int			iEnable;
int			iState;
SCROLLINFO  sInfo;


	if(!m_iInit)return;

	sInfo.fMask		= SIF_PAGE|SIF_POS|SIF_RANGE;
	sInfo.cbSize	= sizeof(sInfo);
	sInfo.nTrackPos	= 0;
	sInfo.nMin		= 0;
	sInfo.nMax		= m_iCount-1;
	sInfo.nPos		= m_iSelect;
	sInfo.nPage		= 1;

	 					     iEnable=ESB_ENABLE_BOTH;
	if(m_iSelect  <=0       )iEnable=ESB_DISABLE_LEFT;
	if(m_iSelect+1>=m_iCount)iEnable=ESB_DISABLE_RIGHT;
	if(m_iCount   <=1       )iEnable=ESB_DISABLE_BOTH;


	iState  = MAKELONG(sInfo.nPos,sInfo.nMax);

	if(iState!=m_iLastButtons)
		{
		m_iLastButtons = iState;
		m_sButtons.SetScrollInfo(&sInfo);						// Scrollbar sperren
		}

	if(iEnable!=m_iLastEnable)
		{
		m_iLastEnable = iEnable;
		// if(m_sButtons.EnableScrollBar(iEnable))				// REM AZ Fehlerhaftes verhalten der Scrollbar
		}


}

//*****************************************************************************
//*
//*		GetScrollBarCtrl
//*
//*****************************************************************************
CScrollBar* CLabelCtrl::GetScrollBarCtrl( int nBar )
{
return (nBar==SB_HORZ)? &m_sScroll:0;
}

//*****************************************************************************
//*
//*		NewName
//*
//*****************************************************************************
//	Namen erzeugen
TCHAR *CLabelCtrl::NewName(const TCHAR *pNamePtr,UINT uLabel)
{
TCHAR  *pName;
TCHAR  *pOld;
int		iLen;


	if(!pNamePtr)												// Standard Namen erzeugen
		{
		pName  = new TCHAR[16];
		memcpy(pName,_T("Fenster "),8*sizeof(TCHAR));
		_itot((uLabel&0xFF)+1,pName+8,10);
		}
	else{														// bergebenen Namen
		iLen  = (int)_tcslen(pNamePtr)+1;
		pName = new TCHAR[iLen];
		memcpy(pName,pNamePtr,iLen*sizeof(TCHAR));
		}

	if(uLabel<(UINT)m_iCount)									// Namen eintragen
		{
		pOld  = m_pTextList[uLabel];
		m_pTextList[uLabel] = pName;
		if(pOld)delete pOld;
		}


return pName;
}


//*****************************************************************************
//*
//*		NewCount
//*
//*****************************************************************************
int	CLabelCtrl::NewCount(UINT uLabels)
{
CWnd	  **pWndListOld;
CWnd	  **pWndListNew;
TCHAR	  **pTextListOld;
TCHAR	  **pTextListNew;
LabelCb    *pSubProcsOld;
LabelCb    *pSubProcsNew;
int		   *pCalHeightNew;
int		   *pCalHeightOld;
int		   *pTextOldX;
int		   *pTextNewX;
int		   *pWidthOld;
int		   *pWidthNew;
int			i;



	if(uLabels>CLW_MAX        )uLabels=CLW_MAX;
	if(uLabels<0              )uLabels=0;
	if(uLabels==(UINT)m_iCount)return uLabels;




	pWndListNew		= new  CWnd  *[uLabels];
	pTextListNew	= new  TCHAR *[uLabels];
	pSubProcsNew	= new  LabelCb[uLabels];
	pCalHeightNew	= new  int 	  [uLabels];
	pTextNewX		= new  int    [uLabels];
	pWidthNew		= new  int    [uLabels];

	pWidthOld		= m_pWidths;
	pTextOldX		= m_pTextSizes;
	pCalHeightOld	= m_pCalHeight;
	pSubProcsOld	= m_pSubProcs;
	pTextListOld	= m_pTextList;
	pWndListOld		= m_pWndList;


	for(i=0;i<(int)uLabels && i<m_iCount;i++)					// Alte Werte kopieren
		{
		pTextNewX    [i] = m_pTextSizes[i];
		pWidthNew    [i] = m_pWidths   [i];
		pCalHeightNew[i] = m_pCalHeight[i];
		pWndListNew  [i] = m_pWndList  [i];
		pTextListNew [i] = m_pTextList [i];
		pSubProcsNew [i] = m_pSubProcs [i];
		}

	for(;i<(int)uLabels;i++)									// Neue Werte
		{
		pTextNewX    [i] = 0;
		pWndListNew  [i] = 0;
		pSubProcsNew [i] = 0;
		pCalHeightNew[i] = 0;
		pTextListNew [i] = NewName(0,i|0x8000);
		pWidthNew    [i] = WIDTH_MIN;
		}


	m_iCount		= 0;										// Blcke Tauschen
	m_pWndList		= pWndListNew;
	m_pTextList		= pTextListNew;
	m_pSubProcs		= pSubProcsNew;
	m_pCalHeight	= pCalHeightNew;
	m_pTextSizes	= pTextNewX;
	m_pWidths		= pWidthNew;
	m_iCount		= uLabels;

	if(pTextOldX    )delete pTextOldX;							// Alte Blcke lschen
	if(pWidthOld    )delete pWidthOld;
	if(pWndListOld  )delete pWndListOld;
	if(pTextListOld )delete pTextListOld;
	if(pSubProcsOld )delete pSubProcsOld;
	if(pCalHeightOld)delete pCalHeightOld;



return uLabels;
}

//*****************************************************************************
//*
//*		GetColor
//*
//*****************************************************************************
//	Abfragen einer Farbe
//	uNum	: Ist die Nummer der Farbe
//				CLW_COLOR_WHITE		0x0001		Farbe fr weise Lasche
//				CLW_COLOR_LABELS	0x0000		Farbe fr Laschen
//				CLW_COLOR_TEXT		0x0002		Farbe fr den Text
//				CLW_COLOR_BG		0x0003		Farbe fr den Hintergrund
//				CLW_COLOR_FRAME		0x0004		Farbe fr die Rahmen
//	Ergibt den RGB Wert der Farbe. 0xFFFFFFFF steht fr die Standartfarbe.
UINT CLabelCtrl::GetColor(UINT uNum)
{

	switch(uNum)
		{
	case CLW_COLOR_WHITE:	return m_uColorLabelW;
	case CLW_COLOR_LABELS:	return m_uColorLabels;
	case CLW_COLOR_FRAME:	return m_uColorFrame;
	case CLW_COLOR_TEXT:	return m_uColorText;
	case CLW_COLOR_BG:		return m_uColorBg;
		}

return 0xFFFFFFFF;
}

//*****************************************************************************
//*
//*		SetColor
//*
//*****************************************************************************
//	Einstellen einer Farbe
//	uNum	: Ist die Nummer der Farbe
//				CLW_COLOR_WHITE		0x0001		Farbe fr weise Lasche
//				CLW_COLOR_LABELS	0x0000		Farbe fr Laschen
//				CLW_COLOR_TEXT		0x0002		Farbe fr den Text
//				CLW_COLOR_BG		0x0003		Farbe fr den Hintergrund
//				CLW_COLOR_FRAME		0x0004		Farbe fr die Rahmen
//	uColor	: Ist der neue Farbwert. 0xFFFFFFFF steht fr die Standartfarbe.
//	Ergibt den vorherigen RGB Wert der Farbe. 0xFFFFFFFF steht fr die Standartfarbe.
UINT CLabelCtrl::SetColor(UINT  uNum,COLORREF uColor)
{
COLORREF	uOld;


	switch(uNum)
		{
	case CLW_COLOR_WHITE:	uOld=m_uColorLabelW;	m_uColorLabelW=uColor;	break;
	case CLW_COLOR_LABELS:	uOld=m_uColorLabels;	m_uColorLabels=uColor;	break;
	case CLW_COLOR_FRAME:	uOld=m_uColorFrame;		m_uColorFrame =uColor;	break;
	case CLW_COLOR_TEXT:	uOld=m_uColorText;		m_uColorText  =uColor;	break;
	case CLW_COLOR_BG:		uOld=m_uColorBg;		m_uColorBg	  =uColor;	break;
	default:				return 0xFFFFFFFF;
		}


	if(uOld!=uColor)
		{
		CreateSplitter();
		RedrawWindow();
		}


return uOld;
}

//*****************************************************************************
//*
//*		CalcTextWidht
//*
//*****************************************************************************
int	CLabelCtrl::CalcTextWidht(const TCHAR *pText)
{
CSize			sSize;
int				iLen;
int				iNum;
HDC			    hDc;


	if(!pText)return 0;


	iNum = (int)_tcslen(pText);

	if(IsWindow(m_hWnd))
			hDc  = ::GetDC(m_hWnd);
	else	hDc  = ::GetDC(0);

	::SelectObject(hDc,m_hTextFont);
	::GetTextExtentPoint(hDc,pText,iNum,&sSize);
	::ReleaseDC(m_hWnd,hDc);

	   iLen = sSize.cx;
	if(iLen>200)iLen=200;


return iLen+m_iSplitterWidth+TEXT_ADD;
}


//*****************************************************************************
//*
//*		CalcLevelPos
//*
//*****************************************************************************
//	Berechnet den Offset der ersten sichtbaren Lasche
void CLabelCtrl::CalcLevelPos(BOOL bRepaint)
{
int		iAllPos;
int		iSelPos;
int		iEndPos;
int		iOffset;
int		iWidth;
int		iSize;
int		iDiff;
int		iAdd;
int		iPos;



	if(m_iCount<=0)												// Keine Laschen ?
		{
		iOffset = 0;
		goto End;
		}

	iDiff	= m_iStartLabel;
	iSize	= m_iLabelWidth-m_iLabelsPosX-WIDTH_MIN;			// Verfgbare Breite fr Laschen
	iOffset	= m_iLabelsPosX-m_iStartOffset;						

	iAllPos	= 0;
	iSelPos	= 0;
	iWidth	= 0;

	if(iSize<0)iSize=0;
	
	for(iPos=0;iPos<m_iCount;iPos++)							// Absolute Offsets berechnen
		{
		iAdd	 = m_iSplitterWidth;
		iAdd	+= m_pWidths[iPos];

		if(iDiff>0)
			{
			iOffset += iAdd;
			iDiff	-= 1;
			}
		
		if(iPos==m_iSelect)
			{
			iWidth	 = iAdd+m_iSplitterWidth;
			iEndPos  = iAllPos+iWidth;
			iSelPos  = iAllPos;
			}
			
		iAllPos += iAdd;
		}					  
	

	iAllPos += m_iSplitterWidth;
	iAllPos += WIDTH_MIN;	


	if(iWidth)													// Ist eine Lasche ausgewhlt
		{
		if(m_iSelect==0)										// Erster Einterag ausgewhlt
			{
			iOffset = 0;
			goto End;
			}

		if(iSize<iWidth)										// Lasche ist grer als Fenster
			{
			iDiff	= (iWidth-iSize)/2;
			iWidth 	=  iSize;
			}
		else{
			   iDiff = (iSize-iWidth)/2;
			if(iDiff>m_iSplitterWidth*2)						// Genug Platz fr Lasche
					iDiff  = m_iSplitterWidth;
			else	iDiff /= 2;

			iSelPos	-= iDiff;
			iEndPos	+= iDiff;
			}
		
		if(iOffset+iSize<iEndPos)
			{
			iOffset = iEndPos-iSize;
			}
		
		if(iOffset>iSelPos)
			{
			iOffset = iSelPos;
			}
		}

End:


	if(iOffset+iSize>iAllPos)									// Zu weit nach links gescrollt
		{
		iOffset = iAllPos-iSize;
		}
	
	if(iOffset<0)												// Zu weit nach rechts gescrollt
		{
		iOffset = 0;
		}


	
	for(iPos=0;iPos<m_iCount;iPos++)							// Startlasche suchen
		{
		iAdd  = m_pWidths[iPos];
		iAdd += m_iSplitterWidth;

		if(iOffset<iAdd)
			{
			break;
			}
		
		iOffset -= iAdd;
		}


	iAdd = m_iLabelsPosX-iOffset;


	if(iAdd!=m_iStartOffset || iPos!=m_iStartLabel)				// Hats sich etwas gendert
		{
		m_iStartLabel  = iPos;
		m_iStartOffset = iAdd;

		if(m_iInit && bRepaint)RedrawWindow();
		}


}


//*****************************************************************************
//*
//*		OnSize
//*
//*****************************************************************************
void CLabelCtrl::OnSize(UINT nType,int iSizeX,int iSizeY)
{
int		iLast;
int		iSize;
int		iOld;
int		i=0;



	iOld = m_iLabelWidth;
	
																	
	if(!(m_iFlags&CLW_SCROLLBAR) || iSizeX<=m_iScrollWidth)		// Keine rechte Scrollbar oder Fenster zu schmal
		{
		m_iLabelWidth = iSizeX;
		}
	else{
		if(m_iScrollHalf)
			{
			m_iLabelWidth = iSizeX/2;
			}

		if(m_iLabelWidth>iSizeX-m_iScrollWidth-WIDTH_SMOVE)
			{
			m_iLabelWidth = iSizeX-m_iScrollWidth-WIDTH_SMOVE;
			}

		if(m_iLabelWidth<m_iScrollWidth)
			{
			m_iLabelWidth = m_iScrollWidth;
			}
		}

	if(m_iSizeX!=iSizeX || iOld!=m_iLabelWidth)					// Position der ersten sichtbaren Lasche berechnen
		{
		CalcLevelPos(0);
		}


							     m_sRectLabel.SetRect(0,-1,iSizeX,iSizeY);
	if(~m_iFlags&CLW_FRAMETOP   )m_sRectLabel.top   --;
	if(~m_iFlags&CLW_FRAMEBOTTOM)m_sRectLabel.bottom++;
	if(~m_iFlags&CLW_FRAMERIGHT )m_sRectLabel.right ++;
	if( m_iFlags&CLW_FRAMELEFT  )i=1;

	iSize		  = (m_iFlags&CLW_NOSTEPBAR)? 0:m_iScrollWidth;
	m_iLabelsPosY = (m_iFlags&CLW_FRAMETOP )? 1:0;
	m_iLabelsPosX = i+iSize;

	if(m_iInit)
		{
		iLast  = m_iScrollWidth;
		iLast ^= iSize			<<14;
		iLast ^= m_iLabelsPosY  <<28;
		iLast ^= i<<30;

		if(iLast!=m_iLastButtonsPos)
			{
			m_iLastButtonsPos = iLast;
			m_sButtons.MoveWindow(i,m_iLabelsPosY,iSize,m_iScrollHeight);
			}

		m_sScroll.MoveWindow(m_iLabelWidth+WIDTH_SMOVE,m_iLabelsPosY,iSizeX-m_iLabelWidth-WIDTH_SMOVE,m_iScrollHeight);
		SetButtons();
		}

	m_iSizeX = iSizeX;
	m_iSizeY = iSizeY;

	if(m_iFlags&CLW_CHILD)
		{
		AttachWnd();											// Kein Child Fenster aktiv ?
		}


}


//*****************************************************************************
//*
//*		OnSysColorChange
//*
//*****************************************************************************
void CLabelCtrl::OnSysColorChange()
{


	CWnd::OnSysColorChange();


	CreateSplitter();


}

//*****************************************************************************
//*
//*		OnTimer
//*
//*****************************************************************************
void CLabelCtrl::OnTimer(UINT_PTR nIDEvent)
{
	if(nIDEvent==TIMER_ID)UpdateChildPos();
	if(nIDEvent==TIMER_BT)
		{
		m_iLastButtons=-1;
		KillTimer(TIMER_BT);
		SetButtons();
		m_sButtons.InvalidateRect(NULL,TRUE);
		}
}

//*****************************************************************************
//*
//*		OnHScroll
//*
//*****************************************************************************
void CLabelCtrl::OnHScroll(UINT nSBCode,UINT nPos,CScrollBar *pScrollBar)
{
SCROLLINFO	sInfo;
unsigned 	uStyle;
int			iStart;
int			iOld;
int			iId;



	iId=GetId(pScrollBar);

	if(iId==SCROLL_SEL)
		{
		iStart = 0;
		m_iLastButtons=-1;
		if(nSBCode==SB_LINELEFT ){SetSel(m_iSelect-1,0);iStart=1;}
		if(nSBCode==SB_LINERIGHT){SetSel(m_iSelect+1,0);iStart=1;}
		if(iStart)SetTimer(TIMER_BT,150,NULL);
		}

	if(iId==SCROLL_SCR && (m_iFlags&CLW_CHILD) && m_pChildWnd && IsWindow(m_pChildWnd->m_hWnd))
		{
		     uStyle=m_pChildWnd->GetStyle();
		if(!(uStyle&WS_HSCROLL))return;

		m_pChildWnd->GetScrollInfo(SB_HORZ,&sInfo);
		m_pChildWnd->GetScrollPos (SB_HORZ);
		iOld = sInfo.nPos;

		switch(nSBCode)
			{
		case SB_LINELEFT:		sInfo.nPos--;				break;
		case SB_LINERIGHT:		sInfo.nPos++;				break;
		case SB_PAGELEFT: 		sInfo.nPos-=sInfo.nPage;	break;
		case SB_PAGERIGHT:		sInfo.nPos+=sInfo.nPage;	break;
		case SB_THUMBPOSITION:	sInfo.nPos =nPos;			break;
		case SB_THUMBTRACK:		sInfo.nPos =nPos;			break;
		case SB_LEFT:			sInfo.nPos--;				break;
		case SB_RIGHT:			sInfo.nPos++;				break;
			}


		switch(m_iScrollMode)
			{
		case 0:	m_pChildWnd->SetScrollInfo(SB_HORZ,&sInfo);
				m_pChildWnd->SendMessage(WM_HSCROLL,MAKELONG(nSBCode,sInfo.nPos),NULL);	break;
		case 1:	((CListCtrl*)m_pChildWnd)->Scroll(CSize(sInfo.nPos-iOld,0));			break;
			}


		m_sScroll.SetScrollInfo(&sInfo);
		}

}

//*****************************************************************************
//*
//*		OnDestroy
//*
//*****************************************************************************
void CLabelCtrl::OnDestroy( )
{
	if(m_iFlags&CLW_CHILD)KillTimer(TIMER_ID);
	DetachWnd();
	m_iInit=0;
	Destroy();

}


//*****************************************************************************
//*
//*		OnMouseMove
//*
//*****************************************************************************
void CLabelCtrl::OnMouseMove(UINT nFlags,CPoint sPoint)
{
int i;

	if(m_iScrollMove)
		{
		i=sPoint.x+m_iLabelWidth-m_sScrollPoint.x;
		if(i>m_iSizeX-WIDTH_SMOVE-m_iScrollWidth)i=m_iSizeX-WIDTH_SMOVE-m_iScrollWidth;
		if(i<m_iScrollWidth    )i=m_iScrollWidth;
		m_sMoveWnd.MoveWindow(i,m_iLabelsPosY,WIDTH_SMOVE,m_iScrollHeight);
		return;
		}

	if(m_iFlags&CLW_SCROLLBAR)									// Breiten Cursor
	if(sPoint.y< m_iLabelsPosY+m_iScrollHeight)
	if(sPoint.x>=m_iLabelWidth)
	if(sPoint.x< m_iLabelWidth+WIDTH_SMOVE)
		{
		SetCursor(m_hCursorSize);
		return;
		}

	SetCursor(m_hCursorArrow);
}

//*****************************************************************************
//*
//*		OnLButtonUp
//*
//*****************************************************************************
void CLabelCtrl::OnLButtonUp(UINT nFlags, CPoint sPoint)
{
int		i;


	if(m_iScrollMove)											// Scrollbreite einstellen
		{
		m_iScrollMove = 0;
		m_sMoveWnd.ShowWindow(SW_HIDE);
		ReleaseCapture();										// Capture freigeben

		i = sPoint.x+m_iLabelWidth-m_sScrollPoint.x;

		if(i>m_iSizeX-WIDTH_SMOVE-m_iScrollWidth)i=m_iSizeX-WIDTH_SMOVE-m_iScrollWidth;
		if(i<m_iScrollWidth    )i=m_iScrollWidth;
		if(  m_iLabelWidth  ==i)return;

		m_iLabelWidth  = i;
		m_iScrollHalf  = 0;

		CalcLevelPos(0);
		OnSize(0,m_iSizeX,m_iSizeY);
		RedrawWindow();

		if(m_iCount && m_pWndList[m_iSelect])
			{
			m_pWndList[m_iSelect]->RedrawWindow();
			}
		}

}

//*****************************************************************************
//*
//*		OnLButtonDown
//*
//*****************************************************************************
void CLabelCtrl::OnLButtonDown(UINT nFlags,CPoint sPoint)
{
int iNum,iOff,iEnd,iWidth;


	if(sPoint.x<m_iLabelsPosX)return; 							// Druck vor erster Lasche
	if(sPoint.y<m_iLabelsPosY)return; 							//			*

	if(m_iFlags&CLW_SCROLLBAR)									// Scroll Breite einstellen
	if(sPoint.y< m_iLabelsPosY+m_iScrollHeight)
	if(sPoint.x>=m_iLabelWidth)
	if(sPoint.x< m_iLabelWidth+WIDTH_SMOVE)
		{
		SetCapture();
		m_iScrollMove=1;
		m_sScrollPoint=sPoint;
		m_sMoveWnd.ShowWindow(SW_SHOW);

		return;
		}

	iNum  = m_iStartLabel;										// Erstes gezeichnete Lasche
	iOff  = m_iStartOffset;										// Position des ersten Lasche
	iOff += m_iSplitterWidth/2;

	if(sPoint.x<iOff)											// Druck vor erster Lasche
		{
		SetSel((iNum>0)? iNum-1:0);
	 	return;
		}

	for(;iNum<m_iCount;iNum++,iOff+=iWidth)						// Lasche suchen
		{
		iWidth  = m_pWidths[iNum];
		iWidth += m_iSplitterWidth;

		   iEnd =iOff+iWidth;
		if(iEnd<=sPoint.x)continue;								// Zur nchsten Lasche

		SetSel(iNum);
		break;
		}

}

//*****************************************************************************
//*
//*		OnRButtonDown
//*
//*****************************************************************************
void CLabelCtrl::OnRButtonDown(UINT nFlags,CPoint sPoint)
{
CWnd   *pWnd;
CRect	sRect;


	if(sPoint.x<m_iLabelsPosX)return; 							// Druck vor erster Lasche
	if(sPoint.y<m_iLabelsPosY)return; 							//			*

	OnLButtonDown(nFlags,sPoint);
	OnLButtonUp  (nFlags,sPoint);

	if(m_iFlags&CLW_NORCLICK)return;
	if((UINT)m_iSelect>=(UINT)m_iCount)return;

		pWnd = m_pWndList[m_iSelect];
	if(!pWnd)return;
	
	pWnd->GetClientRect(&sRect);
	ClientToScreen(&sPoint);	
	pWnd->ScreenToClient(&sPoint);

	if(sPoint.x<=sRect.left  )sPoint.x=sRect.left  +1;
	if(sPoint.x>=sRect.right )sPoint.x=sRect.right -1;
	if(sPoint.y<=sRect.top   )sPoint.y=sRect.top   +1;
	if(sPoint.y>=sRect.bottom)sPoint.y=sRect.bottom-1;

	pWnd->PostMessage(WM_RBUTTONDOWN,nFlags,MAKELONG(sPoint.x,sPoint.y));
	pWnd->PostMessage(WM_RBUTTONUP  ,nFlags,MAKELONG(sPoint.x,sPoint.y));


}

//*****************************************************************************
//*
//*		GetWnd
//*
//*****************************************************************************
//	Das Fenster auf der Position uLabel abfragen.
CWnd *CLabelCtrl::GetWnd(UINT uLabel)
{

	if(uLabel>=(UINT)m_iCount)return 0;

return m_pWndList[uLabel];
}

//*****************************************************************************
//*
//*		GetWndId
//*
//*****************************************************************************
//	Das erste Fenster mit der uId-Nummer uId
CWnd *CLabelCtrl::GetWndId(UINT uId)
{
UINT    uNum;
int		i;


	for(i=0;i<m_iCount;i++)
		{
		if(!m_pWndList[i])continue;

		   uNum= GetId(    m_pWndList[i]);
		if(uNum==uId)return m_pWndList[i];
		}



return 0;
}

//*****************************************************************************
//*
//*		SetWidth
//*
//*****************************************************************************
//	Stellt die Breite einer Lasche ein.
//	uLabel		: Ist die nummer der Lasche
//	iWidth		: Ist die Breite der Lasche, bei 0 wird die Textbreite benutzt.
//	Ergibt die alte Breite bzw. -1 bei einem Fehler
int	CLabelCtrl::SetWidth(UINT uLabel ,int iWidth ,BOOL bRedraw)
{
int iOld;



	if(uLabel>=(UINT)m_iCount || iWidth<0)
		{
		return -1;
		}

	iOld = m_pTextSizes[uLabel];
		   m_pTextSizes[uLabel] = iWidth;

	if(iWidth       ==0)iWidth=CalcTextWidht(m_pTextList[uLabel]);
	if(iWidth<WIDTH_MIN)iWidth=WIDTH_MIN;						// Minimale Breite
	if(iWidth>WIDTH_MAX)iWidth=WIDTH_MAX;						// Maximale Breite

	if(m_iInit && bRedraw && iWidth!=m_pWidths[uLabel])
		{
		RedrawWindow();
		}

	m_pWidths[uLabel]=iWidth;



return iOld;
}


//*****************************************************************************
//*
//*		SetWidth
//*
//*****************************************************************************
//	Stellt die Breite aller Laschen ein.
//	iWidth		: Ist die neue Breite fr die Laschen.
//				  Bei 0 wird die Textbreite benutzt.
//	Ergibt die alte Breite der ersten Lasche bzw. -1 bei einem Fehler
int	CLabelCtrl::SetWidth(int iWidth)
{
int		i,iOld,iRedraw=0;


	if(m_iCount<=0)return -1;

	iOld = m_pTextSizes[0];

	for(i=0;i<m_iCount;i++)
		{
		if(m_pTextSizes[0]==iWidth)continue;
		SetWidth(i,iWidth,0);
		iRedraw=1;
		}

	if(m_iInit && iRedraw)
		{
		RedrawWindow();
		}


return iOld;
}

//*****************************************************************************
//*
//*		SetWidth
//*
//*****************************************************************************
//	Stellt die Breite aller Laschen aus dem einem Feld ein
//	iCount		: Ist die Anzahl der Eintrge im Feld.
//	pWidths		: Ist ein Zeiger auf das Feld mit den Laschenbreiten.
//				  Bei 0 so wird die Textbreite gennommen.
//	bRedraw		: Soll das Fenster neu gezeichnet werden.
//	Ergibt die alte Breite der 1.Lasche bzw. -1 bei einem Fehler
int	CLabelCtrl::SetWidth(int iCount,const int *pWidths,BOOL bRedraw)
{
int		i,iOld,iFlag=0;



	if(!pWidths || m_iCount<=0)return -1;

	iOld = m_pTextSizes[0];

	for(i=0;i<m_iCount && iCount>0;i++,iCount--)
		{
		if(m_pTextSizes[0]==pWidths[i])continue;
		SetWidth(i,pWidths[i],0);
		iFlag = 1;
		}

	if(m_iInit && bRedraw && iFlag)
		{
		RedrawWindow();
		}


return iOld;
}

//*****************************************************************************
//*
//*		GetWidth
//*
//*****************************************************************************
//	Liet die Breite einer Lasche aus.
//	uLabel		: Ist die Nummer der Lasche.
//	Ergibt die Breite der Lasche in Pixel, bzw. -1 bei einem Fehler.
int	CLabelCtrl::GetWidth(UINT uLabel)
{


	if(uLabel>=(UINT)m_iCount)return -1;


return m_pTextSizes[uLabel];
}


//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	Liest einen Laschentext aus.
//	uLabel		: Ist die Nummer der Lasche.
//	Ergibt 0  bei einem Fehler.
LPCTSTR CLabelCtrl::GetText(UINT uLabel)
{


	if(uLabel>=(UINT)m_iCount)return 0;


return m_pTextList[uLabel];
}


//*****************************************************************************
//*
//*		SetText
//*
//*****************************************************************************
//	Stellt den Text fr eine Lasche ein
//	uLabel		: Ist die Nummer der Lasche
//	pText		: Ist der Zeiger auf den neuen Text.
//				  Bei 0 wird ein Standarttext erzeugt.
//	bRedraw		: Soll das Fenster neu gezeichnet werden.
//	Ergibt 0 bzw. -1 bei einem Fehler
int	CLabelCtrl::SetText(UINT uLabel,LPCTSTR pText,BOOL bRedraw)
{

	if(uLabel>=(UINT)m_iCount)return -1;


	NewName(pText,uLabel);										// Neuen Laschenname einstellen

	if(!m_pTextSizes[uLabel])									// Tetxbreite benutzen ?
		{
		m_pWidths[uLabel]=CalcTextWidht(m_pTextList[uLabel]);
		}


	if(m_iInit && bRedraw)RedrawWindow();



return 0;
}

//*****************************************************************************
//*
//*		SetText
//*
//*****************************************************************************
//	Stellt den Text fr alle Laschen ein
//	pChain		: Ist eine String-Kette "Text1\0Text...\0\0"
//	bRedraw		: Soll das Fenster neu gezeichnet werden.
//	Ergibt 0 bzw. -1 bei einem Fehler
int	CLabelCtrl::SetText(LPCTSTR *pChain,BOOL bRedraw)
{
LPCTSTR	 pText;
int		 i;


	if(!pChain || m_iCount<0)return -1;

	pText = *pChain;

	for(i=0;i<m_iCount;i++)
		{
		if(!*pText)break;
		SetText(i,pText,0);
		}


	if(m_iInit && bRedraw)
		{
		RedrawWindow();
		}


return 0;
}


//*****************************************************************************
//*
//*		SetText
//*
//*****************************************************************************
//	Stellt den Text fr alle Laschen gleich ein.
//	Jede Lasche erhlt dann eine aufsteigende ID-Nummer
//	pText		: Zeiger auf den Text fr die Laschen.
//	bRedraw		: Soll das Fenster neu gezeichnet werden.
//	Ergibt 0 bzw. -1 bei einem Fehler
int	CLabelCtrl::SetText(LPCTSTR pText,BOOL bRedraw)
{
int		 i,iLen;
TCHAR	 cBuffer[64];


	if(!pText || m_iCount<0)return -1;

	   iLen=(int)_tcslen(pText);
	if(iLen>58)iLen=58;

	memcpy(cBuffer,pText,iLen*sizeof(TCHAR));

	if(iLen && cBuffer[iLen-1]!=' ')
		{
		cBuffer[iLen]=' ';
		iLen++;
		}

	for(i=0;i<m_iCount;i++)
		{
		_itot(i+1,cBuffer+iLen,10);
		SetText(i,cBuffer,0);
		}

	if(m_iInit && bRedraw)
		{
		RedrawWindow();
		}


return 0;
}

//*****************************************************************************
//*
//*		SetText
//*
//*****************************************************************************
//	Stellt den Text fr alle Laschen ein
//	pTextList	: Ist ein Zeiger Feld auf die Namen
//	bRedraw		: Soll das Fenster neu gezeichnet werden.
//	Ergibt 0 bzw. -1 bei einem Fehler
int	CLabelCtrl::SetText(int iCount,LPCTSTR *pTextList,BOOL bRedraw)
{
int		 i;


	if(!pTextList || m_iCount<0)return -1;

	for(i=0;i<m_iCount && iCount>=0;i++,iCount--)
		{
		if(!pTextList[i])continue;
		SetText(i,pTextList[i],0);
		}


	if(m_iInit && bRedraw)
		{
		RedrawWindow();
		}



return 0;
}

//*****************************************************************************
//*
//*		GetLabel
//*
//*****************************************************************************
//	Das erste Fenster mit der uId-Nummer uId
//	Ergibt -1 wenn uId nicht gefunden wurde
int CLabelCtrl::GetLabel(UINT uId)
{
int		i;
UINT    uNum;


	for(i=0;i<m_iCount;i++)
		{
		if(!m_pWndList[i])continue;

		   uNum= GetId(m_pWndList[i]);
		if(uNum==uId)return i;
		}


return -1;
}

//*****************************************************************************
//*
//*		GetLabel
//*
//*****************************************************************************
//	Sucht ein das Fenster in der Fensterliste.
//	pWnd		: Ist das Fenster nach dem gesucht werden soll.
//	Ergibt -1 wenn uId nicht gefunden wurde
int CLabelCtrl::GetLabel(CWnd *pWnd)
{
int		i;


	for(i=0;i<m_iCount;i++)
		{
		if(!m_pWndList[i]     )continue;
		if( m_pWndList[i]==pWnd)return i;
		}


return -1;
}

//*****************************************************************************
//*
//*		SetWindow
//*
//*****************************************************************************
//	Trgt ein Fenster in die Fensterliste ein.
//	uLabel		: Ist die Nummer der zugehrigen Lasche.
//	pWnd		: Ist das Fenster das eingefgt werden soll.
//	pProc		: Ist die Callback funktion fr das Fentster (0=keine)
//	Ergibt 0 bzw -1 bei einem Fehler
int CLabelCtrl::SetWindow(UINT uLabel,CWnd &sWnd,LabelCb pProc)
{
TCHAR	 cClass[64];


	if(uLabel>=(UINT)m_iCount)return -1;

	m_pWndList [uLabel] = &sWnd;
	m_pSubProcs[uLabel] = pProc;

	cClass[0]=0;
	GetClassName(sWnd.m_hWnd,cClass,sizeof(cClass));

	if(!_tcscmp(cClass,_T("Edit")))
			Calibrate(uLabel,2);
	else	Calibrate(uLabel,0);


	if(uLabel==(UINT)m_iSelect)
		{
		AttachWnd();
		SendNotify();
		}


return 0;
}

//*****************************************************************************
//*
//*		SetWindow
//*
//*****************************************************************************
//	Trgt mehrere Fenster in die Fensterliste ein.
//	iCount		: Ist die Anzahl der Fenster im der Liste.
//	pWndList	: Ist eine Liste mit den einzufgenden Fenstern.
//	pProc		: Ist die Callback funktion fr die Fentster (0=keine)
//	Ergibt 0 bzw -1 bei einem Fehler.
int CLabelCtrl::SetWindow(int iCount,CWnd *pWndList,LabelCb pProc)
{
int		i;


	if(iCount<       0)return -1;
	if(iCount>m_iCount)iCount=m_iCount;

	for(i=0;i<iCount;i++)
		{
		SetWindow(i,pWndList[i],pProc);
		}


return 0;
}


//*****************************************************************************
//*
//*		SetWindow
//*
//*****************************************************************************
//	Trgt mehrere Fenster in die Fensterliste ein.
//	iCount		: Ist die Anzahl der Fenster
//	pWndList	: Ist eine Liste mit den einzufgenden Fenstern
//	pProc		: Ist die Callback funktion fr das Fentser (0=keine)
//	Ergibt 0 bzw -1 bei einem Fehler
int CLabelCtrl::SetWindow(int iCount,CWnd **pWndList,LabelCb pProc)
{
int i;

	if(m_iCount<0)return -1;

	for(i=0;i<m_iCount && iCount>0;i++,iCount--)
		{
		SetWindow(i,*pWndList[i],pProc);
		}


return 0;
}


//*****************************************************************************
//*
//*		SetFlags
//*
//*****************************************************************************
//	Stellt die Ausgabe der Linien ein.
//	iFlags		: Sind die neuen Flags
//	Ergibt die alten StyleFlags
int CLabelCtrl::SetFlags(int iFlags)
{

	if(m_iFlags==iFlags)return iFlags;

	if((m_iFlags&CLW_CHILD) && ((m_iFlags^iFlags)&CLW_CHILD))
		{
		DetachWnd();
		}

	m_iFlags = iFlags;

	if((m_iFlags^iFlags)&(CLW_NOSTEPBAR|CLW_FRAMEALL))
		{
		OnSize(0,m_iSizeX,m_iSizeY);
		}

	if(m_iInit)RedrawWindow();


return m_iFlags;
}


//*****************************************************************************
//*
//*		SetCount
//*
//*****************************************************************************
//	Stellt die Anzahl der Laschen ein.
//	uLabels		: Ist die neue Anzahl der Laschen
//	bRedraw		: Soll das Fenster neu gezeichnet werden.
//	Ergibt die alte Anzahl
int CLabelCtrl::SetCount(UINT uLabels,BOOL bRedraw)
{
int iOld=m_iCount;


	if(uLabels==(UINT)iOld   )return iOld;
	if(uLabels <(UINT)0      )return iOld;
	if(uLabels >(UINT)CLW_MAX)uLabels=CLW_MAX;

	if(uLabels<=(UINT)m_iSelect)SetSel((UINT)0);

	NewCount(uLabels);

	if(m_iInit && bRedraw)RedrawWindow();



return iOld;
}


//*****************************************************************************
//*
//*		SetSel
//*
//*****************************************************************************
//	Whlt eine Lasche aus.
//	uLabel		: Ist die Nummer der zu whlenden Lasche.
//	uUpdate		: Set-Bar auch aktualisieren
//	Ergibt die zuvor gewhlte Laschennummer.
int CLabelCtrl::SetSel(UINT uLabel,UINT uUpdate)
{
CWnd		   *pWndN;
CWnd		   *pWndA;
int				iOld;


	iOld = m_iSelect;

	if(uLabel==(UINT)iOld    )return iOld;
	if(uLabel>=(UINT)m_iCount)return iOld;

	if(iOld>=0 && iOld<m_iCount)
			pWndA = m_pWndList[m_iSelect];
	else	pWndA = NULL;

	pWndN     = m_pWndList[uLabel];
	m_iSelect = uLabel;


	if(m_iFlags&CLW_CHILD)
		{
		DetachWnd();											// Fenster abdocken
		AttachWnd();											// Fenster andocken
		Update();
		}

	if(pWndN)pWndN->ShowWindow(SW_SHOW);
	if(pWndA)pWndA->ShowWindow(SW_HIDE);

	if(m_iInit)													// Fenster zeichnen/benachrichten
		{
		CalcLevelPos(0);
		RedrawWindow();
		SendNotify();
		}

	if(uUpdate)SetButtons();



return iOld;
}


//*****************************************************************************
//*
//*		SendNotify
//*
//*****************************************************************************
//	Sendet eine Notify-Nachricht an das ELternfenster
void CLabelCtrl::SendNotify()
{
LabelNotify		sNotify;
CWnd           *pWnd;


	if(!m_iInit)return;

	if((UINT)m_iSelect<(UINT)m_iCount)
		{
		pWnd = m_pWndList[m_iSelect];
		}
	else{
		pWnd = 0;
		}

	sNotify.hdr.code		= CLW_SELCHANDED;
	sNotify.hdr.hwndFrom	= m_hWnd;
	sNotify.hdr.idFrom		= GetId  (this);
	sNotify.param			= GetData(this);
	sNotify.select			= m_iSelect;

	if(pWnd && IsWindow(pWnd->m_hWnd))
		{
		sNotify.selwnd		= pWnd->m_hWnd;
		sNotify.selid		= (int)GetData(pWnd);
		}
	else{
		sNotify.selwnd		= NULL;
		sNotify.selid		= 0;
		}

	GetParent()->SendMessage(WM_NOTIFY,sNotify.hdr.idFrom,(LPARAM)&sNotify);

}



//*****************************************************************************
//*
//*		Update
//*
//*****************************************************************************
void CLabelCtrl::Update()
{
SCROLLINFO	sInfo;


	if(!m_pChildWnd)return;
	if(!IsWindow(m_pChildWnd->m_hWnd))return;

	sInfo.fMask  = SIF_ALL;
	sInfo.cbSize = sizeof(sInfo);

	m_pChildWnd->GetScrollInfo(SB_HORZ,&sInfo);
	m_sScroll   .SetScrollInfo(        &sInfo);

	if(m_pChildWnd)m_pChildWnd->SendMessage(WM_LUPDATE);

}

//*****************************************************************************
//*
//*		DetachWnd
//*
//*****************************************************************************
//	Aktuelles Fenster abdocken vom Child Bereich
void CLabelCtrl::DetachWnd()
{
SCROLLINFO  sInfo;


	if(m_pChildWnd==0)return;									// Ist Fenster ok

	if(!IsWindow(m_pChildWnd->m_hWnd))
		{
		m_pChildWnd=0;
		return;
		}

	if((m_iFlags&CLW_SCROLLBAR) && m_iLabelWidth>m_iLabelsPosX)	// Scrollbar-Breite merken
		{
		m_iScrollOld = m_iLabelWidth;
		}

	SetStyle(m_pChildWnd,m_iChildStyle);
	m_pChildWnd->MoveWindow(m_sRectChild,FALSE);


	sInfo.fMask		= SIF_PAGE|SIF_POS|SIF_RANGE;				// Scrollbar sperren
	sInfo.cbSize	= sizeof(sInfo);
	sInfo.nTrackPos = 0;
	sInfo.nMin 		= 0;
	sInfo.nMax 		= 0;
	sInfo.nPos 		= 0;
	sInfo.nPage		= 0;

	m_sScroll.SetScrollInfo(&sInfo);

	SetWndProc(m_pChildWnd,m_sWndData.pWndProc);				// Alte Funktion wiederherstellen

	m_sWndData.pWnd		= 0;
	m_sWndData.hWnd		= 0;
	m_sWndData.iStop	= 0;									// Ist die Fensterfunktion bereits aufgerufen
	m_sWndData.pWndProc	= 0;
	m_sWndData.pSubProc = 0;
	m_pChildWnd			= 0;

}

//*****************************************************************************
//*
//*		AttachWnd
//*
//*****************************************************************************
//	Ein Fenster andocken in den Child Bereich
void CLabelCtrl::AttachWnd()
{
CWnd	*pWnd;
TCHAR	 cClass[64];



	if(m_iSelect<0 || m_iSelect>=m_iCount)						// Falsche Auswahl ?
		{
		m_iCalibrate = 0;
		DetachWnd();
		return;
		}

	pWnd = (m_iCount)? m_pWndList[m_iSelect]:0;

	if(m_iFlags & CLW_CHILDSCROLLOFF)							// Scrollbar einschalten
	if(m_iScrollOff && !pWnd && m_iInit)
		{
		m_iScrollOff  = 0;
		m_iLabelWidth = m_iScrollOld;
		OnSize(0,m_iSizeX,m_iSizeY);
		}

	if(m_pChildWnd && m_pChildWnd!=pWnd)						// Altes Fenste abdocken
		{
		DetachWnd();
		}


	m_iCalibrate = m_pCalHeight[m_iSelect];

	if(!(m_iFlags&CLW_CHILD))return; 							// Keine Child-Fenster

	if(m_pChildWnd==pWnd)return;								// Anderses Fenster ?

		 m_pChildWnd  =pWnd;
		 m_iChildStyle=pWnd->GetStyle();
	if(!(m_iChildStyle&WS_CLIPSIBLINGS))SetStyle(pWnd,m_iChildStyle|WS_CLIPSIBLINGS);


	SetWindowPos(&CWnd::wndTop,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);

	cClass[0]=0;
	GetClassName(m_pChildWnd->m_hWnd,cClass,sizeof(cClass));

	if(!_tcscmp(cClass,_T("SysListView32")))
			m_iScrollMode=1;
	else	m_iScrollMode=0;



	if(!IsWindow(pWnd->m_hWnd))return;							// Ist Fenster Ok ?

	m_sWndData.pWnd		= pWnd;
	m_sWndData.hWnd		= pWnd->m_hWnd;
	m_sWndData.iStop	= 0;
	m_sWndData.pWndProc	= GetWndProc(pWnd);
	m_sWndData.pSubProc = m_pSubProcs[m_iSelect];

	SetWindowLongPtr(pWnd->m_hWnd,GWL_USERDATA,(LPARAM)&m_sWndData);
	SetWndProc(pWnd,WindowProc);								// Neue Fensterfunktion einstellen
	Update();

}

//*****************************************************************************
//*
//*		WindowProc
//*
//*****************************************************************************
LRESULT CALLBACK CLabelCtrl::WindowProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
LRESULT		lRet;
WNDMEM	   *pData;
CLabelCtrl *pLabel;
SCROLLINFO  sInfo1;
SCROLLINFO  sInfo2;
unsigned	uStyle;
WNDPROC	    pProc;
int			iDiff;
int			iNew;
int			iId;


	iNew	= 0;
	pData   = (WNDMEM*)GetWindowLongPtr(hWnd,GWL_USERDATA);
	pProc	= pData->pWndProc;
	pLabel	= pData->pLabelCtrl;


	if(uMsg==WM_DESTROY)										// Fenster zersteuren
		{
		SetWndProc(hWnd,pProc);
		pData->pWndProc = 0;
		pData->pSubProc = 0;
		pData->hWnd     = 0;
		pData->pWnd		= 0;
		}


	if(pData->iStop)
		{
		if(pData->pSubProc)
				return pData->pSubProc(pProc,hWnd,uMsg,wParam,lParam,pData->pWnd);
		else	return CallWindowProc (pProc,hWnd,uMsg,wParam,lParam);
		}

	if( uMsg>=WM_LBUTTONDOWN)									// 0x0201 - 0x0209
	if( uMsg<=WM_MBUTTONDBLCLK)iNew=1;							// Maus Tasten
	if((uMsg==WM_PAINT		  )||								// Neu zeichnen
	   (uMsg==WM_NOTIFY		  )||								// Notify-Meldung
	   (uMsg==WM_TIMER        )||								// Timer
	   (uMsg==WM_KEYDOWN      )||								// Tastendruck
	   (uMsg==WM_KEYUP        )||								// Tastendruck
	   (uMsg==WM_LUPDATE      ))iNew=1;							// Fenster erneuern
	if((uMsg==WM_SHOWWINDOW   )|| 								// Fenster Sichtbar machen
	   (uMsg==WM_ERASEBKGND   )||								// Hintergrund lschen
	   (uMsg==WM_SIZE         ))iNew=2;							// Fenstergre ndern
	if( uMsg==WM_HSCROLL      )									// Fenster scrollen
		{
		   iId =GetId((HWND)lParam);
		if(iId==SB_HORZ)iNew=1;
		}

	if(uMsg==WM_NCCALCSIZE)
		{
		NCCALCSIZE_PARAMS	*pParam=(NCCALCSIZE_PARAMS*)lParam;

		uStyle = GetWindowLong(hWnd,GWL_STYLE);

		if(uStyle&WS_HSCROLL)
		if(pLabel->m_iFlags&CLW_CHILDSCROLLOFF)					// Gre der Scrollbar korrigieren
			{
			if(pData->pSubProc)
					lRet = pData->pSubProc(pProc,hWnd,uMsg,wParam,lParam,pData->pWnd);
			else	lRet = CallWindowProc (pProc,hWnd,uMsg,wParam,lParam);

			iDiff  = pParam->rgrc[0].bottom-pParam->rgrc[0].top;
			iDiff -= pLabel->m_sRectChild.Height()+pLabel->m_iCalibrate;

			if(iDiff)
				{
				pParam->rgrc[0].bottom -= iDiff;
				}

			return lRet;
			}
		}

	if(!iNew)
		{
		if(pData->pSubProc)
				return pData->pSubProc(pProc,hWnd,uMsg,wParam,lParam,pData->pWnd);
		else	return CallWindowProc (pProc,hWnd,uMsg,wParam,lParam);
		}

	pData->iStop	= 1;

	if(pData->pSubProc)
			lRet = pData->pSubProc(pProc,hWnd,uMsg,wParam,lParam,pData->pWnd);
	else	lRet = CallWindowProc (pProc,hWnd,uMsg,wParam,lParam);

	sInfo1.fMask=SIF_ALL;sInfo1.cbSize=sizeof(sInfo1);
	sInfo2.fMask=SIF_ALL;sInfo2.cbSize=sizeof(sInfo2);

		  ::GetScrollInfo(hWnd,SB_HORZ,&sInfo1);
	pLabel->GetScrollInfo(     SB_HORZ,&sInfo2);


	if((sInfo1.nPos  != sInfo2.nPos )||							// nderungen ?
	   (sInfo1.nPage != sInfo2.nPage)||
	   (sInfo1.nMax  != sInfo2.nMax )||
	   (sInfo1.nMin  != sInfo2.nMin ))
		{
		pLabel->SetScrollInfo(SB_HORZ,&sInfo1);					// Neue Scroll Daten
		}

	if(iNew==2)pLabel->UpdateChildPos();


	pData->iStop = 0;



return lRet;
}

//*****************************************************************************
//*
//*		UpdateChildPos
//*
//*****************************************************************************
//	berprft ob die Position des angedocken Fensters stimmt
void CLabelCtrl::UpdateChildPos()
{

	if(!(m_iFlags&CLW_CHILD         ))return; 					// Keine Child-Fenster
	if(!(m_iFlags&CLW_SCROLLBAR     ))return;
	if(!m_pChildWnd	                 )return;
	if(!IsWindow(m_pChildWnd->m_hWnd))return;					// Ist Fenster Ok ?


//*****************************************************************************
CRect		sRectC,sRectW;
unsigned 	uStyle;
int			iBorder;
int			iHeight;
int			iWidth;



	m_pChildWnd->GetClientRect(&sRectC);
	m_pChildWnd->GetWindowRect(&sRectW);


	iHeight = m_sRectChild.Height()+m_iCalibrate;
	iWidth  = m_sRectChild.Width ();

	   uStyle=m_pChildWnd->GetStyle();
	if(uStyle&WS_HSCROLL)										// Mit Scrollbar
		{
		if(iHeight!=sRectC.bottom || iWidth!=sRectW.Width())
			{
			m_pChildWnd->ShowWindow(SW_SHOW);
			m_pChildWnd->SetWindowPos(0,m_sRectChild.left,m_sRectChild.top,iWidth,sRectW.Height()-sRectC.bottom+iHeight,SWP_NOZORDER);
			}

		if(m_iFlags&CLW_CHILDSCROLLOFF)
		if(m_iScrollOff)
			{
			m_iScrollOff  = 0;
			m_iLabelWidth = m_iScrollOld;
			OnSize(0,m_iSizeX,m_iSizeY);
			RedrawWindow();
			}
		}
	else{														// Ohne Scrollbar
		iBorder = (uStyle&WS_BORDER)? 1:0;

		if(sRectW.Height()-iBorder!=iHeight || iWidth!=sRectW.Width())
			{
			m_pChildWnd->SetWindowPos(0,m_sRectChild.left,m_sRectChild.top,iWidth,m_sRectChild.bottom+iBorder,SWP_NOZORDER);
			}

		if( m_iFlags&CLW_CHILDSCROLLOFF)
		if(!m_iScrollOff)
			{
			m_iScrollOff = 1;
			m_iScrollOld = m_iLabelWidth;
			OnSize(0,m_iSizeX,m_iSizeY);
			RedrawWindow();
			}
		}

}

//*****************************************************************************
//*
//*		Calibrate
//*
//*****************************************************************************
//	Einstellen der Hhenklibriertung.
//	uLabel		: Bestimmt das Fenster das kalibriert werden soll.
//	iSize		: Ist die Anzahl der Pixel um die das Fenster hher sein soll.
//	Ergibt den vorherigen Wert.
int CLabelCtrl::Calibrate(UINT uLabel,int iSize)
{
int		iOld;


	if(uLabel>=(UINT)m_iCount)return 0;

	iOld = m_pCalHeight[uLabel];

	m_pCalHeight[uLabel] = iSize;


	if(uLabel==(UINT)m_iSelect)
		{
		m_iCalibrate = iSize;
		UpdateChildPos();
		}


return iOld;
}

//*****************************************************************************
//*
//*		CreateSplitter
//*
//*****************************************************************************
//	Erzeugt das Trenn-Bitmap
void CLabelCtrl::CreateSplitter()
{
CDC			   *pDc;
CRgn			sRgn;
UINT			aColors[4];
int				iOffX;
int				iOffY;
int				iPos;
SplitterImage	sImage =
		{
		sizeof(BITMAPINFOHEADER),								// biSize
		19,														// biWidth
		34,														// biHeight
		1,														// biPlanes
		8,														// biBitCount
		BI_RGB,													// biCompression
		20*34,													// biSizeImage
		4705,													// biXPelsPerMeter
		4705,													// biYPelsPerMeter
		30,														// biClrImportant
		30,														// biClrUsed
		};



	if(m_hBitmap)												// Altes Bitmap lschen
		{
		DeleteObject(m_hBitmap);
		m_hBitmap=0;
		}

	if(m_sMemDc.m_hDC)
		{
		m_sMemDc.DeleteDC();
		}

	if(m_uColorLabels==0xFFFFFFFF)
			m_uColorLabUse = GetSysColor(COLOR_3DLIGHT);
	else	m_uColorLabUse = m_uColorLabels;

	if(m_uColorLabelW==0xFFFFFFFF)
			m_uColorLabWin = GetSysColor(COLOR_WINDOW);
	else	m_uColorLabWin = m_uColorLabelW;

	if(m_uColorBg==0xFFFFFFFF)
			m_uColorBgUse = GetSysColor(COLOR_3DLIGHT);
	else	m_uColorBgUse = m_uColorBg;

	if(m_uColorFrame==0xFFFFFFFF)
			m_uColorFrUse = RGB(0,0,0);
	else	m_uColorFrUse = m_uColorFrame;

	if(m_uColorText==0xFFFFFFFF)
			m_uColorTextUse = GetSysColor(COLOR_WINDOWTEXT);
	else	m_uColorTextUse = m_uColorText;


	aColors[0]	= SwapRGB(m_uColorBgUse );
	aColors[1]	= SwapRGB(m_uColorLabUse);
	aColors[2]	= SwapRGB(m_uColorLabWin);
	aColors[3]	= SwapRGB(m_uColorFrUse );

	m_iSplitterHeight =     m_iScrollHeight;
	m_iSplitterWidth  = (   m_iSplitterHeight)/2+2;
	iOffX			  = (35-m_iSplitterHeight)/4;
	iOffY			  = (35-m_iSplitterHeight)/2;

	pDc				  = GetDC();
	m_hBitmap		  = CreateCompatibleBitmap(pDc->m_hDC,m_iSplitterWidth*8,m_iSplitterHeight);

	sRgn    .CreateRectRgn(0,0,m_iSplitterWidth,m_iSplitterHeight);
	m_sMemDc.CreateCompatibleDC(pDc);
	m_sMemDc.SelectObject(m_hBitmap);
	m_sMemDc.SelectObject(sRgn);

	for(iPos=0;iPos<8;iPos++)								// Trenner zeichnen
		{
		sImage.aColors[F20] = aColors[aSectorColors[iPos*4+0]];
		sImage.aColors[F10] = aColors[aSectorColors[iPos*4+1]];
		sImage.aColors[F30] = aColors[aSectorColors[iPos*4+2]];
		sImage.aColors[F40] = aColors[aSectorColors[iPos*4+3]];

		sImage.aColors[F11] = aColors[aLineColors[iPos*6+0]];
		sImage.aColors[F12] = aColors[aLineColors[iPos*6+1]];
		sImage.aColors[F13] = aColors[aLineColors[iPos*6+2]];
		sImage.aColors[F21] = aColors[aLineColors[iPos*6+3]];
		sImage.aColors[F22] = aColors[aLineColors[iPos*6+4]];
		sImage.aColors[F23] = aColors[aLineColors[iPos*6+5]];

		sImage.aColors[F14] = ComineRgb1(sImage.aColors[F11],sImage.aColors[F10]);
		sImage.aColors[F15] = ComineRgb1(sImage.aColors[F12],sImage.aColors[F10]);
		sImage.aColors[F16] = ComineRgb1(sImage.aColors[F13],sImage.aColors[F10]);
		sImage.aColors[F24] = ComineRgb1(sImage.aColors[F21],sImage.aColors[F20]);
		sImage.aColors[F25] = ComineRgb1(sImage.aColors[F22],sImage.aColors[F20]);
		sImage.aColors[F26] = ComineRgb1(sImage.aColors[F23],sImage.aColors[F20]);

		sImage.aColors[F31] = ComineRgb1(sImage.aColors[F11],sImage.aColors[F30]);
		sImage.aColors[F32] = ComineRgb1(sImage.aColors[F21],sImage.aColors[F30]);
		sImage.aColors[F33] = ComineRgb1(sImage.aColors[F13],sImage.aColors[F30]);

		sImage.aColors[F41] = ComineRgb1(sImage.aColors[F12],sImage.aColors[F40]);
		sImage.aColors[F42] = ComineRgb1(sImage.aColors[F22],sImage.aColors[F40]);
		sImage.aColors[F43] = ComineRgb1(sImage.aColors[F13],sImage.aColors[F40]);

		sImage.aColors[F17] = ComineRgb2(sImage.aColors[F11],sImage.aColors[F10]);
		sImage.aColors[F18] = ComineRgb2(sImage.aColors[F12],sImage.aColors[F10]);
		sImage.aColors[F27] = ComineRgb2(sImage.aColors[F21],sImage.aColors[F20]);
		sImage.aColors[F28] = ComineRgb2(sImage.aColors[F22],sImage.aColors[F20]);

		sImage.aColors[F34] = ComineRgb2(sImage.aColors[F11],sImage.aColors[F30]);
		sImage.aColors[F35] = ComineRgb2(sImage.aColors[F21],sImage.aColors[F30]);

		sImage.aColors[F44] = ComineRgb2(sImage.aColors[F12],sImage.aColors[F40]);
		sImage.aColors[F45] = ComineRgb2(sImage.aColors[F22],sImage.aColors[F40]);

		SetDIBitsToDevice(	m_sMemDc.m_hDC,
							m_iSplitterWidth*iPos-iOffX,-iOffY,
							sImage.sHeader.biWidth,
							sImage.sHeader.biHeight,
							0,0,0,
							sImage.sHeader.biHeight,
							aImgData,(BITMAPINFO*)&sImage,
							DIB_RGB_COLORS);

		if((m_iSplitterHeight+1)&3)							// Punkte fr Linien Verlngerung
			{
			if(aLineColors[iPos*6+0]==3)m_sMemDc.SetPixel(m_iSplitterWidth* iPos     ,m_iSplitterHeight-1,m_uColorFrUse);
			if(aLineColors[iPos*6+1]==3)m_sMemDc.SetPixel(m_iSplitterWidth*(iPos+1)-1,m_iSplitterHeight-1,m_uColorFrUse);
			}

		sRgn.OffsetRgn(m_iSplitterWidth,0);
		m_sMemDc.SelectObject(sRgn);
		}


	ReleaseDC(pDc);

}

//*****************************************************************************
//*
//*		OnPaint
//*
//*****************************************************************************
void CLabelCtrl::OnPaint( )
{
CDC		   *pDc;
CRgn		sRgn1;
CRgn		sRgn2;
CRect		sRect;
CRect		sArea;
CSize		sSize;
PAINTSTRUCT sPaint;
int			iHeight;
int			iWidth;
int			iTemp;
int			iMode;
int			iNum;
int			iCnt;
int			iOff;
int			iAdd;


	iNum		 = m_iStartLabel;								// Startpositionen berechnen
	iHeight		 = m_iScrollHeight-1;
	sRect.left	 = m_iStartOffset;
	sRect.top    = m_sRectLabel.top+2;
	sRect.bottom = m_sRectLabel.top+m_iScrollHeight+1;


	sRgn1.CreateRectRgn(m_iLabelsPosX,0,m_iLabelWidth,sRect.bottom+1);

	pDc =BeginPaint  (&sPaint);
	pDc->SetBkColor  (m_uColorLabUse);
	pDc->SetTextColor(m_uColorTextUse);
	pDc->SelectObject(m_hTextFont);
	pDc->SelectObject(GetStockObject(BLACK_PEN));
	pDc->SetBkMode   (OPAQUE);


	if(m_iFlags&CLW_FRAMERIGHT)									// Rahmen rechts
		{
		pDc->MoveTo(m_sRectLabel.right-1,m_sRectLabel.top   );
		pDc->LineTo(m_sRectLabel.right-1,m_sRectLabel.bottom);
		}

	if(m_iFlags&CLW_FRAMELEFT)									// Rahmen links
		{
		pDc->MoveTo(m_sRectLabel.left,m_sRectLabel.top   );
		pDc->LineTo(m_sRectLabel.left,m_sRectLabel.bottom);
		}

	if(m_iFlags&CLW_FRAMETOP)									// Rahmen oben
		{
		pDc->MoveTo(m_sRectLabel.left ,m_iLabelsPosY-1);
		pDc->LineTo(m_sRectLabel.right,m_iLabelsPosY-1);
		}

	if(m_iFlags&CLW_FRAMEBOTTOM)								// Rahmen unten
		{
		pDc->MoveTo(m_sRectLabel.left ,m_sRectLabel.bottom-1);
		pDc->LineTo(m_sRectLabel.right,m_sRectLabel.bottom-1);
		}

	pDc->SelectObject(sRgn1);

	for(;iNum<m_iCount;iNum++)									// Laschen zeichnen
		{
		if(sRect.left>=m_iLabelWidth)break;


		if(iNum-1==m_iSelect)									// Was ist rechts
				iMode  = 2*3;
		else if(iNum>0)
				iMode  = 1*3;
		else	iMode  = 0*3;

		if(iNum==m_iSelect)										// Was ist links
				iMode += 2;
		else	iMode += 1;

		pDc->BitBlt(sRect.left,m_iLabelsPosY,m_iSplitterWidth,m_iScrollHeight,&m_sMemDc,iMode*m_iSplitterWidth,0,SRCCOPY);

		sRect.left	+= m_iSplitterWidth;

		if(sRect.left>=m_iLabelWidth)break;

		iWidth		= m_pWidths[iNum];							// Laschentext zeichnen
		sRect.right = sRect.left+iWidth;
		iCnt		= (int)_tcslen(m_pTextList[iNum]);
		sSize		= pDc->GetOutputTextExtent(m_pTextList[iNum],iCnt);
		iOff		= sRect.left+(iWidth-sSize.cx)/2;

		if(iNum==m_iSelect)
			{
			iAdd	= (m_iFlags&CLW_WHITELINE)? sRect.top-1:sRect.top;
			pDc->SetBkColor(m_uColorLabWin);
			pDc->ExtTextOut(iOff,iAdd,ETO_OPAQUE|ETO_CLIPPED,&sRect,m_pTextList[iNum],iCnt,NULL);

			if(m_iFlags&CLW_WHITELINE)							// Weise Linie
				{
				sArea.left  = sRect.left -m_iSplitterWidth+1;
				sArea.right	= sRect.right+m_iSplitterWidth-1;
				sArea.top    = m_sRectLabel.top+1;
				sArea.bottom = m_sRectLabel.top+2;

				pDc->SetBkColor(m_uColorLabWin);
				pDc->ExtTextOut(0,0,ETO_OPAQUE|ETO_CLIPPED,&sArea,0,0,NULL);
				}
			}
		else{
			pDc->SetBkColor(m_uColorLabUse);
			pDc->ExtTextOut(iOff,sRect.top,ETO_OPAQUE|ETO_CLIPPED,&sRect,m_pTextList[iNum],iCnt,NULL);
			}

		sRect.top    = m_sRectLabel.top+m_iScrollHeight+1;
		sRect.bottom = m_sRectLabel.top+m_iScrollHeight+2;

		pDc->SetBkColor(m_uColorFrUse);
		pDc->ExtTextOut(0,0,ETO_OPAQUE|ETO_CLIPPED,&sRect,0,0,NULL);

		sRect.left	 = sRect.right;
		sRect.top    = m_sRectLabel.top+2;
		sRect.bottom = m_sRectLabel.top+m_iScrollHeight+1;
		}


	if(sRect.left<m_iLabelWidth)								// Abschluss zeichnen
		{
		if(iNum<=0)												// Was ist links
				iMode  = 0*3;
		if(iNum-1!=m_iSelect)
				iMode  = 1*3;
		else	iMode  = 2*3;

		pDc->BitBlt(sRect.left,m_iLabelsPosY,m_iSplitterWidth,m_iScrollHeight,&m_sMemDc,iMode*m_iSplitterWidth,0,SRCCOPY);

		sRect.left	+= m_iSplitterWidth;

		if(sRect.left<m_iLabelWidth)							// Bereich nach Laschen fllen
			{
			sRect.bottom += 1;
			sRect.right   = m_iLabelWidth;
			pDc->SetBkColor(m_uColorBgUse);
			pDc->ExtTextOut(0,0,ETO_OPAQUE|ETO_CLIPPED,&sRect,0,0,NULL);
			}
		}

	sRgn2.CreateRectRgn(0,0,m_iSizeX,m_iSizeY);
	pDc->SelectObject(sRgn2);




	if(sRect.bottom+2<m_sRectLabel.bottom)						// Unteren Abschluss zeichnen
		{
		iTemp		 = sRect.left;
		sRect.left	 = m_sRectLabel.left;
		sRect.right	 = m_sRectLabel.right;
		sRect.top    = m_sRectLabel.top+m_iScrollHeight+2;
		sRect.bottom = m_sRectLabel.bottom;

		if(m_iFlags&CLW_FRAMELEFT  )sRect.left++;					// Rahmen links
		if(m_iFlags&CLW_FRAMERIGHT )sRect.right--;					// Rahmen rechts
		if(m_iFlags&CLW_FRAMEBOTTOM)sRect.bottom--;					// Rahmen unten

		pDc->SetBkColor(m_uColorBgUse);
		pDc->ExtTextOut(0,0,ETO_OPAQUE|ETO_CLIPPED,&sRect,0,0,NULL);
		}


	if(m_iFlags&CLW_SCROLLBAR)									// Scrollbar-Verschubflche
		{
		pDc->SelectStockObject(BLACK_PEN);
		pDc->SelectObject(GetSysColorBrush(COLOR_3DLIGHT));
		pDc->Rectangle(m_iLabelWidth,m_iLabelsPosY-1,m_iLabelWidth+WIDTH_SMOVE,m_iLabelsPosY+m_iScrollHeight);
		}


	EndPaint(&sPaint);

}

