//*****************************************************************************
//*
//*
//*		LabelCtrl.h
//*
//*
//*****************************************************************************
//
//	Funktion:
//
//		Ein Label-Fenster schickt an sein Elternfenster eine WM_NOTIFY
//		Nachricht diese kann ber ON_NOTIFY(CLW_SELCHANDED,uId,MemberFunktion)
//		abgefangen werden.
//
//		MemberFunktion ist vom Typ 
//
//			OnSelChanged(NMHDR *pNmHdr, LRESULT *pResult)
//				{
//				LabelNotify	*pNotify=(LabelNotify*)pNmHdr;
//				...
//				}
//
//			pNotify->hdr.idFrom		 : Ist die ID des LabelCtrl
//			pNotify->selct			 : Ist die neu gewhlte Lasche
//			pNotify->param			 : Ist der Wert von Userdata in LabelCtrl
//			pNotify->selid			 : Ist die neu gewhlte Lasche
//			pNotify->select			 : Ist die Nummer des gerade gewhlte Eintrags
//			pNotify->selwnd			 : Ist das Fenster-Handle des gewhlten Eintrags
//
//		Ist einer Lasche ein Fenster zu geordnet so wird das Fenster
//		sichtbar bzw. unsichtbar geschalten.
//
//
//	Fenster erzeugen:
//
//		m_cLabelCtrl.Create(iScanner,0,"Daten",sRect,this,CLW_FRAMETOP|CLW_CHILDSCROLL|CLW_CHILDSCROLLOFF|CLW_WHITELINE,12345);
//		m_cLabelCtrl.SetChildRect(sRect.left,sRect.top-200,sRect.right,sRect.top);
//		m_cLabelCtrl.SetWindow(iScanner,m_aListWnd);
//		m_cLabelCtrl.ShowWindow(SW_SHOW);
//
//	
//	Grennderung des Elternfensters 
//
//		m_cLabelCtrl.SetChildRect(0,0     ,iSizeX,       iWidth);
//		m_cLabelCtrl.MoveWindow  (0,iWidth,iSizeX,iSizeY-iWidth);
//		m_cLabelCtrl.UpdateChildPos();
//
//
//	Funktionen:
//
// 		int   GetSel      ()						: Liest aktuelle Auswahl
// 		int	  SetSel      (UINT     uLabel)			: Setzt aktuelle Auswahl
// 		int	  SetSel      (CWnd    *pWnd  )			: Setzt aktuelle Auswahl mit CWnd Zeiger
// 		int	  GetSelId    (UINT     uId   )			: Liest ID des aktuellen Fensters
// 		int	  SetSelId    (UINT 	uId   )			: Setzt aktuelle Auswahl mit ID
// 		int	  SetWidth    (int      iWidth)			: Laschenbreiten auf einmal setzen
// 		int	  SetWidth    (int,int  iWidth,BOOL)	: Einzelne Laschenbreite setzen
// 		int	  SetWidth    (int,int *iWidth,BOOL)	: Laschenbreiten ber Feld setzen
// 		int	  GetWidth    (UINT     uLabel  )		: Laschenbreite lesen
// 		int	  GetLabel    (UINT     uId     )		: Lasche mit uId finden
// 		int	  GetLabel    (CWnd    *pWnd    )		: Lasche mit Fenster finden
// 		int	  GetCount    (                 )		: Anzahl der Laschen lesen
// 		int	  SetCount    (int         ,BOOL)		: Anzahl der Laschen setzen
// 		char *GetText     (UINT       uLabel)		: Alle Laschennamen lesen
// 		int	  SetText     (char      **,BOOL)		: Alle Laschennamen setzen mit Kette
// 		int	  SetText     (char       *,BOOL)		: Alle Laschennamen setzen
// 		int	  SetText     (int,char  **,BOOL)		: Alle Laschennamen setzen mit Feld
// 		int	  SetText     (int,char   *,BOOL)  		: Einen Laschennamen setzen
// 		CWnd *GetWnd      (UINT  uLabel     )		: Fenster mit Nummer finden
// 		CWnd *GetWndId    (UINT	 uId        )		: Fenster mit uId finden
// 		int	  SetWindow   (int,CWnd    &pWnd)		: Einer Lasche ein Fenster zuordnen
// 		int	  SetWindow   (int,CWnd    *pWnd)		: Allen Laschen Fenster zuordnen
// 		int	  SetWindow   (int,CWnd   **pWnd)		: Allen Laschen Fenster zuordnen
// 		int	  SetFlags    (int iFlags       )		: Zeichenmodus fr Lasche
// 		void  SetChildRect(RECT &sRect      )		: Child-Bereich einstellen
// 		void  SetChildRect(int,int,int,int  ) 		: Child-Bereich einstellen
// 		void  Update      (                 )		: Scrollzustand Updaten


#ifndef	  __LABELCTRL_H__
#define   __LABELCTRL_H__

#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#define		VC_EXTRALEAN	// Selten verwendete Teile der Windows-Header nicht einbinden
#include	<Afxwin.h>      // MFC-Kern- und -Standardkomponenten
#include	<Afxext.h>      // MFC-Erweiterungen
#include	<Afxcmn.h>		// MFC-Untersttzung fr gngige Windows-Steuerelemente
#include	"WinExtension.h"

#if		   _MSC_VER >= 1000
#pragma		once
#endif



typedef	struct
	{
	NMHDR	hdr;
	LPARAM	param;												// Ist der Userdata-Parameter vom Fester
	int 	selid;												// Ist die Fenster-ID des gewhlten Eintrags	
	int		select;												// Ist der gerade gewhlte Eintrag
	HWND	selwnd;												// Ist das Fenster-Handle des gewhlten Eintrags
	}LabelNotify;


typedef LRESULT	(CALLBACK *LabelCb)(WNDPROC,HWND,UINT,WPARAM,LPARAM,CWnd*);

//*****************************************************************************
//*
//*		Konstanten
//*
//*****************************************************************************

#define 	CLW_MAX				32								// Maximale Auswahlmglichkeit.
#define 	CLW_ID				0x0000ABAB						// LabelCtrl Standard ID.
#define 	CLW_WHITELINE		0x0001							// Eine Weise Linie zeichnen.
#define 	CLW_SCROLLBAR		0x0002							// H-Scrollbar einblenden.
#define 	CLW_NOSTEPBAR		0x0004							// Die Stepbar ausblenden.
#define 	CLW_NORCLICK		0x0008							// Die Rechte Maustaste nicht weiterleiten
#define 	CLW_FRAMETOP		0x0010							// Rahmen Optionen
#define 	CLW_FRAMEBOTTOM		0x0020							//			*
#define 	CLW_FRAMELEFT		0x0040							//			*
#define 	CLW_FRAMERIGHT		0x0080							//			*
#define 	CLW_FRAMEALL   		0x00F0							//			*
#define 	CLW_CHILD      		0x0100							// Automatisches platzieren der Kindfenster 
#define 	CLW_CHILDSCROLLOFF	0x0200							// Scrollbar abschaltbar
#define 	CLW_CHILDSCROLL		0x0302							// Child mit Scrollbar abschalten
#define 	CLW_SELCHANDED		0x0001							// Notifynachricht bei Auswahlnderung
#define 	CLW_COLOR_LABELS	0x0000							// Farbe fr Laschen
#define 	CLW_COLOR_WHITE		0x0001							// Farbe fr weise Lasche
#define 	CLW_COLOR_TEXT		0x0002							// Farbe fr den Text
#define 	CLW_COLOR_BG		0x0003							// Farbe fr den Hintergrund
#define 	CLW_COLOR_FRAME		0x0004							// Farbe fr die Rahmen



//*****************************************************************************
//*
//*		Klasse
//*
//*****************************************************************************
class CLabelCtrl : public CWnd
{

public:

		     CLabelCtrl();
	virtual ~CLabelCtrl();

	BOOL	Create		(UINT uLabels,int  iWidth,LPCTSTR  pText,            const RECT &sRect,CWnd *pParentWnd,DWORD dwStyle=WS_CHILD|WS_CLIPCHILDREN|WS_VISIBLE,UINT nID=CLW_ID);
	BOOL	Create		(UINT uLabels,int  iWidth,LPCTSTR *pText,            const RECT &sRect,CWnd *pParentWnd,DWORD dwStyle=WS_CHILD|WS_CLIPCHILDREN|WS_VISIBLE,UINT nID=CLW_ID);
	BOOL	Create		(UINT uLabels,int *pWidth,LPCTSTR *pText,            const RECT &sRect,CWnd *pParentWnd,DWORD dwStyle=WS_CHILD|WS_CLIPCHILDREN|WS_VISIBLE,UINT nID=CLW_ID);
	BOOL	Create		(UINT uLabels,int  iWidth,LPCTSTR  pText,CWnd **pWnd,const RECT &sRect,CWnd *pParentWnd,DWORD dwStyle=WS_CHILD|WS_CLIPCHILDREN|WS_VISIBLE,UINT nID=CLW_ID);
	BOOL	Create		(UINT uLabels,int  iWidth,LPCTSTR *pText,CWnd **pWnd,const RECT &sRect,CWnd *pParentWnd,DWORD dwStyle=WS_CHILD|WS_CLIPCHILDREN|WS_VISIBLE,UINT nID=CLW_ID);
	BOOL	Create		(UINT uLabels,int *pWidth,LPCTSTR *pText,CWnd **pWnd,const RECT &sRect,CWnd *pParentWnd,DWORD dwStyle=WS_CHILD|WS_CLIPCHILDREN|WS_VISIBLE,UINT nID=CLW_ID);

	int 	GetSel		(            );
	int		SetSel	 	(UINT  uLabel);
	int		SetSel	 	(CWnd *pWnd  );
	int		SetSelId 	(UINT  uId   );
	int		GetWidth 	(UINT  uLabel);
	int		SetWidth 	(int   iWidth);
	int		SetWidth 	(UINT  uLabel ,      int      iWidth,BOOL bRedraw=TRUE);
	int		SetWidth 	(int   iCount ,const int     *pWidth,BOOL bRedraw=TRUE);
	int		SetCount 	(UINT  uLabels                      ,BOOL bRedraw=TRUE);
	int		GetCount 	(            );
	LPCTSTR	GetText  	(UINT  uLabel);
	int		SetText  	(LPCTSTR                     *pChain,BOOL bRedraw=TRUE);
	int		SetText  	(LPCTSTR                      pText ,BOOL bRedraw=TRUE);
	int		SetText  	(int   iCount ,      LPCTSTR *pText ,BOOL bRedraw=TRUE);
	int		SetText  	(UINT  uLabel ,      LPCTSTR  pText ,BOOL bRedraw=TRUE);
	int		GetLabel 	(UINT	uId   );
	int		GetLabel 	(CWnd *pWnd  );
	CWnd   *GetWnd   	(UINT  uLabel);
	CWnd   *GetWndId 	(UINT	uId   );
	int		Calibrate	(UINT  uLabel ,int   iSize);
	int		SetWindow	(UINT  uLabel ,CWnd  &sWnd,LabelCb=0);
	int		SetWindow	(int   iCount ,CWnd  *pWnd,LabelCb=0);
	int		SetWindow	(int   iCount ,CWnd **pWnd,LabelCb=0);
	int		SetFlags 	(int   iFlags);
	int		SetData  	(int   iData );
	UINT	GetColor	(UINT  uNum);
	UINT	SetColor	(UINT  uNum,COLORREF uColor);   
	void 	SetChildRect(RECT &sRect);
	void 	SetChildRect(int iPosX,int iPosY,int iSizeX,int iSizeY);
	void	UpdateChildPos();
	void	Update();

	int		GetScrollLimit(int nBar												   );
	int		GetScrollPos  (int nBar												   );
	BOOL	GetScrollInfo (int nBar,LPSCROLLINFO pScrollInfo ,UINT  nMask = SIF_ALL);
	void	GetScrollRange(int nBar,LPINT        pMinPos     ,LPINT pMaxPos        );
	int 	SetScrollPos  (int nBar,int          nPos        ,BOOL  bRedraw = TRUE );
	BOOL	SetScrollInfo (int nBar,LPSCROLLINFO pScrollInfo ,BOOL  bRedraw = TRUE );
	void	SetScrollRange(int nBar,int nMinPos, int nMaxPos ,BOOL  bRedraw = TRUE );

	virtual CScrollBar *GetScrollBarCtrl(int nBar=SB_HORZ);

protected:

	BOOL	Create		  (UINT uLabels,int *pWidth,LPCTSTR *pText,CWnd **pWnd,const RECT &sRect,CWnd *pParentWnd,DWORD dwStyle=WS_CHILD|WS_CLIPCHILDREN|WS_VISIBLE,UINT nID=CLW_ID,int iMode=0xFF);
	int		NewCount      (UINT uLabels);
	TCHAR  *NewName       (const TCHAR *pName,UINT uLabel);
	int		CalcTextWidht (const TCHAR *pText);
	int		SetSel	 	  (UINT  uLabel,UINT uUpdate);
	void	CalcLevelPos  (BOOL  bRepaint);
	void	CreateSplitter();
	void	SendNotify    ();
	void	SetButtons    ();
	void	AttachWnd     ();
	void	DetachWnd     ();
	void	Destroy       ();


	static	LRESULT CALLBACK WindowProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);


	//{{AFX_MSG(CLabelCtrl)
	afx_msg void OnPaint();
	afx_msg void OnDestroy();
	afx_msg void OnSysColorChange();
	afx_msg void OnTimer      (UINT_PTR nIDEvent);
	afx_msg void OnSize       (UINT nType, int iSizeX, int iSizeY);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint sPoint);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint sPoint);
	afx_msg void OnLButtonUp  (UINT nFlags, CPoint sPoint);
	afx_msg void OnMouseMove  (UINT nFlags, CPoint sPoint);
	afx_msg void OnHScroll    (UINT nSBCode,UINT nPos,CScrollBar *pScrollBar );
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()

//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************

protected:

	CScrollBar		m_sButtons;									// Nummern Tasten
	CScrollBar		m_sScroll;									// Scrollbar
	CStatic			m_sMoveWnd;									// Scrollbar verschieben
	CWnd		  **m_pWndList;									// Zeiger auf Fenster
	CWnd		   *m_pChildWnd;								// Zeiger auf Child-Fenster
	CWnd		   *m_pChildParent;								// Zeiger auf Child-Elternfenster
	LabelCb  	   *m_pSubProcs;								// Callbacks fr Fenster
																
	int				m_iSelect;									// Aktuelle ausgewhlte Lasche
	int				m_iFlags;									// Zeichen Modi
	int				m_iSizeX;									// Aktuelle Fenster iWidth
	int				m_iSizeY;									//		*
	int				m_iCalibrate;								// Aktuelle Korrigierte Hhe
	int			   *m_pCalHeight;								// Korrigierte Hhe fr Fenster
	int			   *m_pWidths;									// Feld mit Breiten
	int			   *m_pTextSizes;								// Feld mit Namenbreiten
	TCHAR		  **m_pTextList;								// Feld mit Namen
																
private:														
																
	typedef	struct
		{
		CWnd	   *pWnd;										// Zeiger auf das Fenster	
		HWND		hWnd;										// Fenster Handle
		int			iStop;										// Ist die Fensterfunktion bereits aufgerufen
		WNDPROC		pWndProc;									// Alte Fensterprozedur
		LabelCb     pSubProc;									// Callback fr Fenster
		CLabelCtrl *pLabelCtrl;									// This-Zeiger
		}WNDMEM,*LPWNDMEM;


		CDC			m_sMemDc;									// DC fr den Splitter
		WNDMEM		m_sWndData;									// Speicher fr Fensterdaten
		HBITMAP		m_hBitmap;									// Bitmaps fr Splitter
		COLORREF	m_uColorLabels;								// Farben fr die Laschen (0xFFFFFFFF=default)
		COLORREF	m_uColorLabUse;								// Farben fr die Laschen (zum Zeichen)
		COLORREF	m_uColorLabelW;								// Farben fr die weise Lasche (0xFFFFFFFF=default)
		COLORREF	m_uColorLabWin;								// Farben fr die weise Lasche (zum Zeichen)	
		COLORREF	m_uColorText;								// Farben fr den Text (0xFFFFFFFF=default)
		COLORREF	m_uColorTextUse;							// Farben fr den Text (zum Zeichen)	
		COLORREF	m_uColorBg;									// Farben fr den Hintergrund (0xFFFFFFFF=default)
		COLORREF	m_uColorBgUse;								// Farben fr den Hintergrund (zum Zeichen)	
		COLORREF	m_uColorFrame;								// Farben fr die Linien (0xFFFFFFFF=default)
		COLORREF	m_uColorFrUse;								// Farben fr die Linien (zum Zeichen)	
static	HCURSOR		m_hCursorSize;								// Cursor zum ndern der Scrollbar-Breite
static	HCURSOR 	m_hCursorArrow;								// Normaler Cursor
static	HFONT	 	m_hTextFont;								// Schrift fr Anzeige
static	int			m_iInitDone;								// Anzahl der Fenster
static	int			m_iScrollHeight;							// Hhe   einer Scroll Taste
static	int			m_iScrollWidth;								// Breite einer Scroll Taste
		int			m_iSplitterHeight;							// Hhe des Trenners
		int			m_iSplitterWidth;							// Breite des Trenners
		int			m_iCount;									// Anzahl der Laschen
		int			m_iInit;									// Ist Fenster initialisiert
		int			m_iStartLabel;								// Erstes gezeichneten Lasche
		int			m_iStartOffset;								// Startposition der ersten Lasche (X-Koordinate)
		int			m_iLabelsPosX;								// X-Position der Laschen
		int			m_iLabelsPosY;								// Y-Startposition fr Label-Fenster
		int			m_iScrollMode;								// Ist der modus fr's Fenster
		CRect		m_sRectChild;								// Abmessungen des Kindfensters
		CRect		m_sRectLabel;								// Abmessungen des Laschenfeldes 
		int			m_iLabelWidth;								// Breite fr Label-Fenster
		int			m_iChildStyle;								// Style des Child Fenster
		int			m_iScrollHalf;								// Scrollbar auf halbe Breite
		int			m_iScrollMove;								// Wird Scroll Breite eingesttellt
		int			m_iScrollOff;								// Ist Scrollbar abgeschalten
		int			m_iScrollOld;								// Alte Scrollbar Breite
		int			m_iLastEnable;								// Letzte Einstellungen fr Ebable-Step-Bar
		int 		m_iLastButtons;								// Letzte Einstellungen fr Step-Bar
		int 		m_iLastButtonsPos;							// Letzte Position fr Step-Bar
		CPoint		m_sScrollPoint;								// Scroll Breite Maus Punkt

};


//*****************************************************************************
//*
//*		Inline	Funktionen
//*
//*****************************************************************************

inline int 	CLabelCtrl::GetSel   (          ){return m_iSelect;};
inline int	CLabelCtrl::SetSel	 (UINT  uPos){return SetSel(uPos,1);}
inline int 	CLabelCtrl::SetSel   (CWnd *pWnd){return SetSel(GetLabel(pWnd));}
inline int 	CLabelCtrl::SetSelId (UINT   uId){return SetSel(GetLabel(uId ));}
inline int	CLabelCtrl::GetCount (          ){return m_iCount;};
inline int	CLabelCtrl::SetData  (int  iData){return (int)SetWindowLongPtr(m_hWnd,GWL_USERDATA,iData);}

inline void CLabelCtrl::SetChildRect  (RECT &sRect                                     ){m_sRectChild=sRect;}
inline void CLabelCtrl::SetChildRect  (int iPosX,int iPosY,int iSizeX,int iSizeY       ){m_sRectChild.SetRect(iPosX,iPosY,iPosX+iSizeX,iPosY+iSizeY);}
inline BOOL CLabelCtrl::GetScrollInfo (int nBar, LPSCROLLINFO pScrollInfo,UINT  nMask  ){return GetScrollBarCtrl(nBar)->GetScrollInfo (pScrollInfo,nMask      );};
inline int	CLabelCtrl::GetScrollLimit(int nBar										   ){return GetScrollBarCtrl(nBar)->GetScrollLimit(					      );};
inline int	CLabelCtrl::GetScrollPos  (int nBar										   ){return GetScrollBarCtrl(nBar)->GetScrollPos  (					 	  );};
inline void CLabelCtrl::GetScrollRange(int nBar,LPINT pMinPos           , LPINT pMaxPos){	    GetScrollBarCtrl(nBar)->GetScrollRange(pMinPos,pMaxPos        );};
inline int  CLabelCtrl::SetScrollPos  (int nBar,int nPos                , BOOL  bRedraw){return GetScrollBarCtrl(nBar)->SetScrollPos  (nPos           ,bRedraw);};
inline BOOL CLabelCtrl::SetScrollInfo (int nBar,LPSCROLLINFO pScrollInfo, BOOL  bRedraw){return GetScrollBarCtrl(nBar)->SetScrollInfo (pScrollInfo    ,bRedraw);};
inline void CLabelCtrl::SetScrollRange(int nBar,int nMinPos, int nMaxPos, BOOL  bRedraw){	    GetScrollBarCtrl(nBar)->SetScrollRange(nMinPos,nMaxPos,bRedraw);};

inline BOOL CLabelCtrl::Create(UINT L,int  B,LPCTSTR  T         ,const RECT &R,CWnd *W,DWORD D,UINT I){return Create(L,(int*)B,(LPCTSTR*)T,0,R,W,D,I,0x10);}
inline BOOL CLabelCtrl::Create(UINT L,int  B,LPCTSTR *T         ,const RECT &R,CWnd *W,DWORD D,UINT I){return Create(L,(int*)B,(LPCTSTR*)T,0,R,W,D,I,0x12);}
inline BOOL CLabelCtrl::Create(UINT L,int *B,LPCTSTR *T         ,const RECT &R,CWnd *W,DWORD D,UINT I){return Create(L,(int*)B,(LPCTSTR*)T,0,R,W,D,I,0x13);}
inline BOOL CLabelCtrl::Create(UINT L,int  B,LPCTSTR  T,CWnd **N,const RECT &R,CWnd *W,DWORD D,UINT I){return Create(L,(int*)B,(LPCTSTR*)T,N,R,W,D,I,0x10);}
inline BOOL CLabelCtrl::Create(UINT L,int  B,LPCTSTR *T,CWnd **N,const RECT &R,CWnd *W,DWORD D,UINT I){return Create(L,(int*)B,(LPCTSTR*)T,N,R,W,D,I,0x12);}
inline BOOL CLabelCtrl::Create(UINT L,int *B,LPCTSTR *T,CWnd **N,const RECT &R,CWnd *W,DWORD D,UINT I){return Create(L,(int*)B,(LPCTSTR*)T,N,R,W,D,I,0x13);}


#endif
